#include <string>
#include <cstring>
#include <cmath>
#include <sstream>
#include <fstream>
#include <iostream>
#include <vector>
#include <algorithm>

/*
To compile:
g++ -O3 tax_models.cpp -lm -o tax_models

Tax_models.cpp was written by Donald Jacobs, initially during summer of 2013.
The program was then modified slightly during January 2014.
hold7_tax_models  ==> Is the program that was thought complete after January 2014.
Unfamliarity with the code in December 2015 prompted the program to be documented. 
Bugs were found and many calculations were confusing or questioned. 
December 2015: Bugs fixed, additons made, clean up, results checks & documentation. 
This version was initially backed up as hold8_tax_models.
However, the latest version is called tax_models.cpp , which is slightly different
than hold8_tax_models0

Send correspondence to Dr. Donald Jacobs at UNC Charlotte.
djacobs1@uncc.edu

--------------------------------------------------------------------------------
   QUIRKS
   Some quirks that should be fixed but will remain as inherited from beginning.

1. Earnings is used for market_income 

   Making the appropriate changes in names is not a difficult task but lack of 
   time requires old names to remain in the program. 

--------------------------------------------------------------------------------
   INCOME DISTRIBTIONS
   This program calculates the earnings distribution based on a specified model.

1. Earnings distribution is best defined as market income from US IRS data.
2. Properties of the earnings distribution are part of the program's output.

   Assumptions 
   How is the Market Income (mi) taken from IRS precisely defined?
   A potential problem is that the reported IRS market income is actually 
   the adjusted gross income (AGI) which has deductions in it to begin with. 
   If AGI is used, the tax rate from the victory tax system will be higher
   than reality because it is not accounting for some missing income that 
   was already deducted. However, in this program, the initial distribution 
   is assumed to be without any deductions except for a certain amount of
   capital loss deductions that is handled as:
   market_income = earnings + investment_income 
   investment_income = capital_gains - capital_losses

--------------------------------------------------------------------------------
   TAX MODELS
1. Perfect Progressive Tax model
   Outputs tax characteristics of a perfect progressive tax model.
   This means that the tax rate is a linear function of income percentile. 
   No basic or itemized deductions are allowed in this model.
   Having no deductions preserves the effective tax rate. 
   Capital loss deductions are allowed in same way as in the victory tax.
2. Victory Tax model
   Outputs tax characteristics based on a victory tax model with deductions.
3. Flat Tax model
   Same as victory tax model except with MP=0% => no deductions allowed. 
   As a special case of a victory tax model it has identical type of output.
   Outputs tax characteristics in same formats as the victory tax model.
   REMARK: The victory tax model came as a consequence of working with a 
           true flat tax and seeing that any form of deductions make the
           effective tax rates NOT FLAT, and hence not a flat tax.  

--------------------------------------------------------------------------------
   GOVERNMENT ASSISTANCE
1. Transfer 
   Actual money leaving the government to assist income of household. 
   This transfer is for households that would otherwise be below poverty line.
   This transfer only puts these below-poverty line incomes at the poverty line.
   All models have this (victory tax, flat tax and perfect progressive tax)
2. Deductions 
  *The basic deduction (BD) provides non-uniform tax savings in reality.  
   Here basic deduction is uniform because the effective household size is fixed.
  *The itemized deduction provides non-uniform tax saving, from 0 to MD.
   The maximum deduction (MD) controls how much one can itemize.
   We use only the basic deduction to get the least tax savings. 
   We use the maximum deduction to get the most tax savings. 
   Estimated itemized deductions are also estimated in the DEDUCTION MODEL below.
3. Capital losses 
   Tax savings also come from capital losses on investments. 
   Capital losses are treated differently due to investment income definition. 
   Earnings distribution is assumed to account for capital losses already. 
   Estimated capital loss deductions are estimated in the DEDUCTION MODEL below.
4. government subsidy 
   subsidy = transfer + (deduction + capital_losses)*tax_rate

--------------------------------------------------------------------------------
   DEDUCTION MODEL (updated in Dec 2015)
1. Basic deduction is accounted for in victory tax 
   With only basic deduction, maximum tax rate is determined and reported as ETR. 
   Tax savings is calculated as part of deduction. 
2. Maximum itemized deduction is accounted for in victory tax 
   Assuming maximum deduction, determines minimum tax rate (MTR). 
3. Estimated itemized deduction is variable, which needs a model. 
   Tax savings on itemized deduction is estimated using a linear model.
   nominal_itemized_deduction = 0.5*Earnings*(income_percentile)^2. 
   Note that the maximum deduction constraint will enforce a cutoff. 
4. Estimated capital loss deduction also needs a model. 
   nominal_capital_loss = CL*Earnings*(income_percentile)^4.
   In this formula, CL is a user-defined fraction ranging from 0 to 1. 

--------------------------------------------------------------------------------
   COMPARISON INVARIANT QUANTITIES
   Define deduction schemes relative to population percentiles.

   Using income percentage and earnings_percentiles makes it possible to 
   directly compare various tax results for different income distributions
   and different deduction schemes. 

--------------------------------------------------------------------------------
   TAX REVENUE INPUT
1. -tR => total revenue collected from the population (default, from IRS 2003)
   Different tax systems will collect different totals. 
   Internal variable name for total revenue will be irs_revenue_ 
2. -tT = total tranfer distributed to the population (default, from IRS 2003)
   Different tax systems will redistribute different amount of transfers.
   Internal variable name for total transfer will be irs_transfr_

--------------------------------------------------------------------------------
   KEY QUANTITIES 

  P(E) = Population density (# earners with earnings between E and E + dE).
         Remark 1: P(E) serves as the input to determine effects of a tax model.
         Remark 2: k_function is considered to be the actual market income data. 
                   Model parameters are optimized to fit to market income data.
         Remark 3: Other models are available to provide comparisions. 
                   Models: k_function, log_normal, quadratic, linear, uniform

  E(x) = Earnings density for a rank ordered percent of population by income. 
         Remark 1: Monotonic function: E(x+dx) > E(x) for dx>0. 
         Remark 2: Integral of E(x) from x=0 to 100% gives total earnings.
         Remark 3: Ranked earnings = 100*rank_ordered_earnings(%)/N_earners
                   For specified income percentile => gives earnings. 

  T(x) = Tax density for a rank ordered percent of population by income. 
         Remark 1: T(x) need not be monotonic depending on the deduction model.
         Remark 2: Integral of T(x) from x=0 to 100% gives total tax collected.
         Remark 3: Effective tax rate (in percent) = 100*T(x)/E(x) for each x. 


  See USAGE for details of the type of output that is generated.
  The program runs in three modes: 0, 1, 2.
  0 => fitting. 
  1 => 1 model at a time. 
  2 => run a series of models at a time to make comparisions. 
  
*/

using namespace std;

// GLOBAL VARIABLES

   enum model_t          // model types for population to earnings distributions
{
    unknown,                                                    // unknown model 
    uniform,                                   // defined over entire population
    linear,                                    // defined over entire population
    quadratic,                                 // defined over entire population
    log_normal,               // defined by a normal distribution on a log scale
    k_function                // k-generalized distribution by Clementi, et. al. 
};

// ------------------------------------------------------- specialized functions

double expK( double x, double kappa ) 
{
// *****************************************************************************
   double y;
// *****************************************************************************
   double z = kappa*x;
   double t = z + sqrt( 1.0 + z*z );
   double ikap = 1.0/kappa;
   y = pow( t , ikap ); 
   return y; 
}

double lnK( double x, double kappa ) 
{
// *****************************************************************************
   double y;
// *****************************************************************************
   double z = pow( x , kappa );
   y = (z - 1.0/z)/(2.0*kappa);
   return y; 
}

double gamma_function( double x ) 
{
// *****************************************************************************
   double y,sum,dt;
   double q; 
// *****************************************************************************
   x = (x - 1.0); 
      if( x < -0.99 ) 
      {
      cout << endl;
      cout << "argument in gamma function should be > -0.99: error stop" 
           << endl;
      cout << endl;
      exit(-1); 
      } 
// ------------------------------------------------------------- do the integral
   y = 0.0;
// -------------------------------------------------------------------- stage -4
   dt = 0.00000000001;
   sum = 0.0;
      for(double t=0.0; t<0.00000500000; t+=0.00000000001)
      {
      q = t + 0.5*dt;
      sum += pow( q, x )/exp(q);
      }
   y += sum*dt; 
// -------------------------------------------------------------------- stage -3
   dt = 0.0000000001;
   sum = 0.0;
      for(double t=0.000005; t<0.0000500000; t+=0.0000000001)
      {
      q = t + 0.5*dt;
      sum += pow( q, x )/exp(q);
      }
   y += sum*dt; 
// -------------------------------------------------------------------- stage -2
   dt = 0.000000001;
   sum = 0.0;
      for(double t=0.00005; t<0.0004000000; t+=0.000000001)
      {
      q = t + 0.5*dt;
      sum += pow( q, x )/exp(q);
      }
   y += sum*dt; 
// -------------------------------------------------------------------- stage -1
   dt = 0.00000001;
   sum = 0.0;
      for(double t=0.0004; t<0.001000000; t+=0.00000001)
      {
      q = t + 0.5*dt;
      sum += pow( q, x )/exp(q);
      }
   y += sum*dt; 
// --------------------------------------------------------------------- stage 0
   dt = 0.0000001;
   sum = 0.0;
      for(double t=0.001; t<0.00500000; t+=0.0000001)
      {
      q = t + 0.5*dt;
      sum += pow( q, x )/exp(q);
      }
   y += sum*dt; 
// --------------------------------------------------------------------- stage 1
   dt = 0.000001;
   sum = 0.0;
      for(double t=0.005; t<0.02000000; t+=0.000001)
      {
      q = t + 0.5*dt;
      sum += pow( q, x )/exp(q);
      }
   y += sum*dt; 
// --------------------------------------------------------------------- stage 2
   dt = 0.00001;
   sum = 0.0;
      for(double t=0.02; t<0.4000000; t+=0.00001)
      {
      q = t + 0.5*dt;
      sum += pow( q, x )/exp(q);
      }
   y += sum*dt; 
// --------------------------------------------------------------------- stage 3
   dt = 0.0001;
   sum = 0.0;
      for(double t=0.4; t<8.0000000; t+=0.0001)
      {
      q = t + 0.5*dt;
      sum += pow( q, x )/exp(q);
      }
   y += sum*dt; 
// --------------------------------------------------------------------- stage 4
   dt = 0.001;
   sum = 0.0;
      for(double t=8.0; t<40.0000000; t+=0.001)
      {
      q = t + 0.5*dt;
      sum += pow( q, x )/exp(q);
      }
   y += sum*dt; 
// --------------------------------------------------------------------- stage 5
   dt = 0.01;
   sum = 0.0;
      for(double t=40.0; t<100.0000000; t+=0.01)
      {
      q = t + 0.5*dt;
      sum += pow( q, x )/exp(q);
      }
   y += sum*dt; 

return y; 
}
   

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
class TaxModel 
{
 public: 
 TaxModel() : N_earners_(112117000),                            // for 2003 data
              ave_earnings_(75300),                             // for 2003 data
              irs_revenue_(1952929045000),                      // for 2003 data
              irs_transfr_(1098746600000),                      // for 2003 data
              L_percntile_(40),               // for 2003 (using 1.5 x estimate)
              H_percntile_(50),                // median income seems reasonable
              inc_percent_(60),                          // generous values > 50 
              input_Eend_aid_(-1.0),       // => default indicates not specified
              input_max_deduction_(-1.0),  // => default indicates not specified 
              ON_(1.0),                // default is to tax government transfers
              CL_(0.4),                     // % capital loss relative to profit
              flag_prog_tax_(1),                    // calculate progressive tax
              flag_victory_(1),                         // calculate victory tax
              sigma_(-1.0),         // sigma_ must be > 0. -1.0 => not specified
              alpha_(-1.0),         // alpha_ must be > 0. -1.0 => not specified
              kappa_(-1.0)          // kappa_ must be > 0. -1.0 => not specified
              {name_.assign("out");  
               info_.assign(""); 
               fopt_.assign("opt.in");} 

   void set_nE(double nE) {N_earners_ = nE;}
   void set_aE(double aE) {ave_earnings_ = aE;}
   void set_tR(double tR) {irs_revenue_ = tR;}
   void set_tT(double tt) {irs_transfr_ = tt;}
   void set_BD(double bd) {L_percntile_ = bd;}
   void set_MD(double md) {H_percntile_ = md;}
   void set_CL(double cl) {CL_ = cl;}
   void set_MP(double mp) {inc_percent_ = mp;}
   void set_mi(string model) { model_ = unknown;
                               if( model == "uniform" )   model_ = uniform; 
                               if( model == "linear" )    model_ = linear; 
                               if( model == "quadratic" ) model_ = quadratic; 
                               if( model == "lognormal" ) model_ = log_normal; 
                               if( model == "Kfunction" ) model_ = k_function; 
                             }
   void set_BDd(double bd) {input_Eend_aid_ = bd;}
   void set_MDd(double md) {input_max_deduction_ = md;}
   void set_sig(double sig) {sigma_ = sig;}
   void set_alp(double alp) {alpha_ = alp;}
   void set_kap(double kap) {kappa_ = kap;}
   void set_Nm(char * name) {name_.assign(name);}
   void set_fop(char* name) {fopt_.assign(name);}

   void turnoff_taxing_transfers() {ON_ = 0.0;}
   void do_prog_tax_only() {flag_prog_tax_=1; flag_victory_=0;}
   void do_victoryT_only() {flag_prog_tax_=0; flag_victory_=1;}
   void do_all_tx_models() {flag_prog_tax_=1; flag_victory_=1;}

   void specialize_name(); 
   void update(double & x_target); 

   double calc_ave_etr();

   string get_Nm() {return name_;}
   string get_if() {return info_;}
   string get_fo() {return fopt_;}
   double get_poverty_level() {return poverty_level_;}
   double get_MD() {return MD_;}
   double get_MP() {return inc_percent_;}
   double get_ON() {return ON_;}
   double get_CL() {return CL_;}


 void output(); 

// ------------------------------- methods related to income related information
 void read_tax_data(); 
 bool set_market_income(const int & argc, char **const argv );
 void set_earnings_support();  
 void renormalize();    // for any income distribution 

// ----------------------- Kfunction specific methods
 void calc_Kfunction(); 
 void grid_search(double min_a, double max_a, double min_k, double max_k); 
 double LSE_Kfunction();

// ----------------------- general output & data manipulation methods
 void std_earnings();
 void Exx_earnings();
 void market_income_by_earnings();
 void market_income_by_population();
 void market_income_brackets();

// ------------------------------------------------- methods related to taxation
 void initialize_endaid_line();   // must define end_aid line before doing calcs
 void initialize_tax_revenue();   // set tax revenue and government transfr info
 void calc_prog_tax_revenues();   // sets output file names
 void calc_victoryT_revenues();   // sets output file names 

 void out_effective_tax_rate();
 void out_maximum_tax_rate(); 
 void out_taxes_lorenz_curve(); 
 void out_taxes_paid_brckets();
 void out_paid_by_population();     // <= taxes paid 

 void out_gov_subsidie_ratio();
 void out_g_tr_by_population();       // net transfer = (assistance + deduction) 
 void out_gv_tr_lorenz_curve();       // net transfer = (assistance + deduction) 
 void out_g_transfer_brckets();            // net government transfer (brackets) 

 private: 
 vector<double> earnings_variable_;          // non-uniform independent variable
 vector<double> d_earnings_variable_;     // differences in independent variable
 vector<double> earnings_distribution_;      // probability density for earnings

 vector<double> percentile_variable_;        // non-uniform independent variable
 vector<double> d_percentile_variable_;   // differences in independent variable
 vector<double> rank_ordered_earnings_;    // earnings vs. population percentile
 vector<double> least_tax_density_;  // LEAST tax dnsty vs. population percntile
//              ^^^^^^^^^--------> lowest victory tax rate => maximum deductions
// ----------------------------------------------- poverty elimination liability
 vector<double> transfer_density_;  //transfer density vs. population percentile
//              ^^^^^^^^^^^^^^^^^-------------> only the part for minimal income
// --------------------- ESTIMATED deduction densities vs. population percentile
 vector<double> deduction_density_;        // from basic and itemized deductions 
 vector<double> cptl_loss_density_;           // from capital investiment losses
// --------------------------------- receipt densities vs. population percentile
 vector<double> receipt_T_density_;                 // from government transfers
 vector<double> receipt_D_density_;        // from basic and itemized deductions 
 vector<double> receipt_I_density_;           // from capital Investiment losses

 model_t model_;
 double N_earners_; 
 double ave_earnings_;
 double irs_revenue_;
 double irs_transfr_;
 double L_percntile_;
 double H_percntile_;
 double inc_percent_;          // maximum percent of income that can be deducted
 double input_Eend_aid_;                // maximum government support in dollars
 double input_max_deduction_;                    // maximum deduction in dollars
 double ON_;                  // = (1.0, 0.0) to tax or not government transfers
 double CL_;                    // = capital loss ratio relative to profit [0,1] 
 int flag_prog_tax_;       // (0,1) = (does not, does) calculate progressive tax
 int flag_victory_;            // (0,1) = (does not, does) calculate victory tax
 double sigma_;                            // needed for log normal distribution
 double alpha_;                            // needed for K-function distribution
 double kappa_;                            // needed for K-function distribution
 std::string name_; 
 std::string info_; 
 std::string fopt_; 

 double max_earnings_;
 double min_earnings_;
 double tot_earnings_;
 double std_earnings_;
 double E20_earnings_;                          // earnings at 20% of population
 double E40_earnings_;                          // earnings at 40% of population
 double E50_earnings_;                          // earnings at 50% of population
 double E60_earnings_;                          // earnings at 60% of population
 double E80_earnings_;                          // earnings at 80% of population
 double E99_earnings_;                          // earnings at 99% of population
 double N_Esupport_;                            // defines array length for P(E)
 double dP_;
 double beta_;                             // needed for K-function distribution

// ----------------------------------------------- for optimization of Kfunction
 double target1_; 
 double target2_; 
 double target3_; 
 double target4_; 
 double target5_; 
 double target6_; 

 double Eend_aid_;               // maximum income before no government transfer
 double poverty_level_;                                  // actual poverty level

// ---------------------------------------- tax variables dependent on tax model
 std::string ef_tax_id_;
 std::string lz_tax_id_;
 std::string burden_id_;                                        // <= taxes paid 
 std::string Q6_tax_id_;
 std::string mx_ati_id_;                              // <= max after tax income
 std::string mn_ati_id_;                              // <= min after tax income
 std::string ratio_id_;                         // <= subsidy/(after tax income) 
 std::string kudos_id_;                // <= subsidy = (assistance + deductions)
 std::string mingt_id_;                           // <= minimum subsidy possible
 std::string gt_lz_id_;
 std::string Q6_gt_id_;
 std::string P_etr_id_;                      // Probability for average tax rate
 std::string C_etr_id_;           // Cumulative probability for average tax rate
 double BL_;                                              // BL_ = CL_/(1 - CL_) 
 double BD_;                                      // basic deduction = Eend_aid_
 double MD_;                        // maximum deduction when inc_percent_ = 100
 double Emin_deduction_;          // = (BD_ , 0 ) for (victory, progressive) tax
 double progressive_tax_rate_;    // 40% at 100% => 40/100 = 20/50 = 10/25 = 0.4
 double same_tax_rate_;                                            // fixed rate
 vector<double> tax_rate_;                // tax rate can be flat or progressive
 double tax_revenue_;                         // tax revenue from selected model
 double net_revenue_;         // net tax revenue after transfers: defines target
 double mia_transfr_;                               // minimal income assistance
 double net_subsidy_;                               // mia_transfr_ + deductions
};

void TaxModel::specialize_name()
{
// ----------------------------------------------------------------- setup info_
   std::stringstream s;
// -------------------------------------------------------------------- BD input
      if( input_Eend_aid_ < 0.0 )             // population percentile specified
      {
      int bd = (L_percntile_ + 0.5);
      s << "_BD" << bd; 
      }
      else                                             // dollar value specified
      {
      int end_aid = (input_Eend_aid_ + 0.5);
      s << "_BDd" << end_aid; 
      }
// -------------------------------------------------------------------- MD input
      if( input_max_deduction_ < 0.0 )        // population percentile specified
      {
      int md = (H_percntile_ + 0.5); 
      s << "_MD" << md; 
      }
      else                                             // dollar value specified
      {
      int md = (input_max_deduction_ + 0.5);
      s << "_MDd" << md; 
      }
// -------------------------------------- income percentage is always percentage
   int mp = (inc_percent_ + 0.5); 
   s << "_MP" << mp;                                       // maximum percentage
// -------------------------------- capital loss percentage is always percentage
   int cl = (100.0*CL_ + 0.5); 
   s << "_CL" << cl; 
// -------------------------------------------------- taxing transfers ON or OFF
      if( ON_ > 0.5 )                                                   // => ON
      {
      s << "_ON"; 
      }
      else                                                             // => OFF
      {
      s << "_OFF"; 
      }
   info_ = s.str();
};

double TaxModel::calc_ave_etr()
{
// --------------------------------------------------------------------- purpose
// calculates the average effective tax rate but also the entire distribution 
// for the tax rate. The greater population with a certain effective tax rate 
// will show up here as P(etr). A simple binning scheme is used to perform the
// calculation put P(etr) will be considered a probability density.
// -----------------------------------------------------------------------------
// ---------------- calculate discretized probability distribution for tax rates
 double ave_etr;
 fstream outPetr,outCetr;
 std::string filename;
 filename = name_ + info_ + P_etr_id_;            // probability density for etr
 outPetr.open( filename.c_str() , fstream::out );
 filename = name_ + info_ + C_etr_id_;         // cumulative probability for etr
 outCetr.open( filename.c_str() , fstream::out );
 vector<double> tr;                                   // effective tax rate, ETR
 vector<double> pb;       // probability for ETR within 0.1% accuracy (bin size)
 int Nb = 500;                                     // over a 100% range on [0,1]
 double db = 100.0/Nb;                                            // size of bin
 int nb3 = Nb + 3;
 tr.reserve(nb3);    
 tr.resize(nb3); 
 pb.reserve(nb3);    
 pb.resize(nb3); 
 double pbsum = 0.0;
 double pretax_income_density;
 double effective_tax_ratio;
 double w,cpb;
 int max_bin;
 int bin_num;
 max_bin = -1;

// ------------------------------------------------------------------ population 
//      dtemp = earnings_distribution_.at(i)*d_earnings_variable_.at(i);
//      rank_ordered_earnings_.at(i) = E/scale;                  // total earnings
//      d_percentile_variable_.at(i) = dtemp*scale;

// ----------------------------------------------------------- initialize arrays
      for(int i=0; i<nb3; i++)
      {
      tr[i] = 0;
      pb[i] = 0;
      }
// ------------------------------------------------------ collect data into bins
   ave_etr = 0.0;
      for(int i=0; i<percentile_variable_.size(); i++)
      {
      if( percentile_variable_.at(i) < 0.0 ) break;
      pretax_income_density = rank_ordered_earnings_.at(i) 
                            + transfer_density_.at(i);
      effective_tax_ratio = least_tax_density_.at(i)/pretax_income_density;
      bin_num = floor(Nb*effective_tax_ratio);
      w = d_percentile_variable_.at(i); 
      // w = earnings_distribution_.at(i)*d_earnings_variable_.at(i); // same as d_percentile_variable_.at(i)
      tr[bin_num] += w*effective_tax_ratio; 
      pb[bin_num] += w; 
      ave_etr += w*effective_tax_ratio;
      pbsum += w;
         if( bin_num > max_bin )
         {
         max_bin = bin_num; 
         }
//    cout << i << "  "  << w << "  " << 100*effective_tax_ratio << endl; 
      }
   max_bin += 1;                                  // number of bins counts 0 too
   ave_etr /= pbsum; 
   ave_etr *= 100;
// --------------------------------------------------- normalize bin information
   double dtemp = 1.0/(pbsum*db);
   double dtemp2;
   cpb = 0.0; 
      for(int i=0; i<max_bin; i++)
      {
      dtemp2 = pb[i]*dtemp;              // gives normalized probability density
         if( dtemp2 > 1.0e-8 )
         {
         tr[i] /= pb[i];                                 // average ETR in bin i
         tr[i] *= 100; 
         pb[i] *= dtemp; 
         cpb += db*pb[i]; 
         outPetr << tr[i] << "  " << pb[i] << endl;
         outCetr << tr[i] << "  " << cpb   << endl;
         }
      }
   outPetr.close();
   outCetr.close();
   return ave_etr;
}
void TaxModel::update(double & x_target)
{
 double dx_target; 
    if( (x_target < 0.01) || (x_target > 99.99) )
    {
    dx_target = 0.0001; 
    }
    else if( (x_target < 0.1) || (x_target > 99.9) )
    {
    dx_target = 0.001; 
    }
    else if( (x_target < 1.0) || (x_target > 99.0) )
    {
    dx_target = 0.01; 
    }
    else
    {
    dx_target = 0.1; 
    }
 x_target += dx_target; 
};


void TaxModel::market_income_by_earnings()
{
// ----------------------------------------------------------- setup file output
 std::string filename;
 fstream output;
 filename = name_ + info_ + "_Pmi.inc";   // population density by market income
 output.open( filename.c_str() , fstream::out );
 output << "# tot_earnings = " << tot_earnings_ << endl;
 output << "# max_earnings = " << max_earnings_ << endl;
 output << "# ave_earnings = " << ave_earnings_ << endl;
 output << "# E20_earnings = " << E20_earnings_ << endl;
 output << "# E40_earnings = " << E40_earnings_ << endl;
 output << "# E50_earnings = " << E50_earnings_ << endl;
 output << "# E60_earnings = " << E60_earnings_ << endl;
 output << "# E80_earnings = " << E80_earnings_ << endl;
 output << "# E99_earnings = " << E99_earnings_ << endl;
 output << "# std_earnings = " << std_earnings_ << endl;
 output << "# min_earnings = " << min_earnings_ << endl;
 output << "# num_earners  = " << N_earners_    << endl;
 output << "# ------------------------------- " << endl;

 int skip; 
    if( N_Esupport_ < 5000 ) 
    {
    skip = 0; 
    }
    else if( N_Esupport_ < 10000 ) 
    {
    skip = 1; 
    }
    else if( N_Esupport_ < 30000 ) 
    {
    skip = 3; 
    }
    else if( N_Esupport_ < 50000 ) 
    {
    skip = 5; 
    }
    else 
    {
    skip = 10; 
    }
 double sum = 0.0; 
 int count = 10000000; 
    for(int i=0; i<N_Esupport_; i++)
    {
    count++;
    sum += earnings_distribution_.at(i)*d_earnings_variable_.at(i); 
       if( count > skip )
       {
       count = 0;
       output << earnings_variable_.at(i) << "  " 
              << earnings_distribution_.at(i) << "  " << sum <<  endl;
       } 
    }
 output.close();
};

 void TaxModel::market_income_by_population()
{
// ----------------------------------------------------------- setup file output
 std::string filename;
 fstream output,lorenz;
 filename = name_ + info_ + "_rmi.inc";    // rmi = ranked-ordered market income
 output.open( filename.c_str() , fstream::out );
 filename = name_ + info_ + "_mLz.inc";      // mLz = market income Lorenz curve
 lorenz.open( filename.c_str() , fstream::out );
// -----------------------------------------------------
 output << "# tot_earnings = " << tot_earnings_ << endl;
 output << "# max_earnings = " << max_earnings_ << endl;
 output << "# ave_earnings = " << ave_earnings_ << endl;
 output << "# E20_earnings = " << E20_earnings_ << endl;
 output << "# E40_earnings = " << E40_earnings_ << endl;
 output << "# E50_earnings = " << E50_earnings_ << endl;
 output << "# E60_earnings = " << E60_earnings_ << endl;
 output << "# E80_earnings = " << E80_earnings_ << endl;
 output << "# E99_earnings = " << E99_earnings_ << endl;
 output << "# std_earnings = " << std_earnings_ << endl;
 output << "# min_earnings = " << min_earnings_ << endl;
 output << "# num_earners  = " << N_earners_    << endl;
 output << "# ------------------------------- " << endl;
// -----------------
 double gini = 0.0;
 double sum = 0.0; 
 double dtemp;
 double E; 
 vector<double> x;
 vector<double> yE;
 vector<double> yS;
 vector<double> yL;
 double dx_target; 
 double x_target = -0.0001; 
 double x_deviation; 
    for(int i=0; i<percentile_variable_.size(); i++)
    {
    if( percentile_variable_.at(i) < 0.0 ) break;
    sum += rank_ordered_earnings_.at(i)*d_percentile_variable_.at(i); 
    E = 100.0*rank_ordered_earnings_.at(i)/N_earners_;
//  output << percentile_variable_.at(i) << "  " << E << "  " << sum << endl;
    dtemp = 100.0*sum/tot_earnings_; 
//  lorenz << percentile_variable_.at(i) << "  " << dtemp << endl;
    gini += (percentile_variable_.at(i) - dtemp)*d_percentile_variable_.at(i);
// -------------------------- store subset of data instead of printing all of it
   x_deviation = x_target - percentile_variable_.at(i); 
       if( x_deviation < 0.0 ) 
       {
       x.push_back( percentile_variable_.at(i) );
       yE.push_back(E);
       yS.push_back(sum);
       yL.push_back(dtemp);
       update(x_target); 
          if( x_target < percentile_variable_.at(i) ) 
          {
          x_target = percentile_variable_.at(i) + 0.00001; 
          }
       } 
    }
 x.push_back(100.000000000);
 yE.push_back(E);
 yS.push_back(sum);
 yL.push_back(dtemp);
// -------------------------------------------------------- write subset of data
    for(int i=0; i<x.size(); i++)
    {
    output << x.at(i) << "  " << yE.at(i) << "  " << yS.at(i) << endl;
    lorenz << x.at(i) << "  " << yL.at(i) << endl;
    }
 gini /= 5000.0;
 if( (gini < 0.0) && (gini > -0.025) ) gini = 0.0; 
 lorenz << "# Gini coefficient = " << gini << endl;
 output.close();
 lorenz.close();
};

 void TaxModel::out_taxes_paid_brckets()
{
// ----------------------------------------------------------- setup file output
 std::string filename;                                    // ending with vtQ.tax
 fstream output;
 filename = name_ + info_ + Q6_tax_id_;    // five quintiles and top 1% quantile
 output.open( filename.c_str() , fstream::out );
// -------------------------------------------------------------------- brackets
 double sum_bin1 = 0.0;
 double sum_bin2 = 0.0;
 double sum_bin3 = 0.0;
 double sum_bin4 = 0.0;
 double sum_bin5 = 0.0;
 double sum_bin6 = 0.0;
    for(int i=0; i<percentile_variable_.size(); i++)
    {
    if( percentile_variable_.at(i) < 0.0 ) break;
       if( percentile_variable_.at(i) < 20.0 ) 
       {
       sum_bin1 += least_tax_density_.at(i)*d_percentile_variable_.at(i); 
       }
       else if( percentile_variable_.at(i) < 40.0 ) 
       {
       sum_bin2 += least_tax_density_.at(i)*d_percentile_variable_.at(i); 
       }
       else if( percentile_variable_.at(i) < 60.0 ) 
       {
       sum_bin3 += least_tax_density_.at(i)*d_percentile_variable_.at(i); 
       }
       else if( percentile_variable_.at(i) < 80.0 ) 
       {
       sum_bin4 += least_tax_density_.at(i)*d_percentile_variable_.at(i); 
       }
       else
       {
       sum_bin5 += least_tax_density_.at(i)*d_percentile_variable_.at(i); 
       }
       if( percentile_variable_.at(i) >= 99.0 )
       {
       sum_bin6 += least_tax_density_.at(i)*d_percentile_variable_.at(i); 
       }
    }
// ---------------------------------------------------------------------- output
 double sum = 0.0;
 sum += sum_bin1;
 sum += sum_bin2;
 sum += sum_bin3;
 sum += sum_bin4;
 sum += sum_bin5;
// -------------------------------------------------------------- sanity check 1
   if( fabs(sum - tax_revenue_) > 1000 )      // typically within 1.0e-8 % error
   {
   cout << "ERROR: sanity check 1" << endl;
   cout << "       tax collected is NOT equal to tax_revenue_ " << endl; 
   cout << "sum of taxes collected = " << sum << endl;
   cout << "          tax_revenue_ = " << tax_revenue_ << endl;
   exit(-1);
   }
 output << "# minimum tax revenue collected = " << sum << endl;
 double p1 = 100.0*sum_bin1/sum; 
 double p2 = 100.0*sum_bin2/sum; 
 double p3 = 100.0*sum_bin3/sum; 
 double p4 = 100.0*sum_bin4/sum; 
 double p5 = 100.0*sum_bin5/sum; 
 double p6 = 100.0*sum_bin6/sum; 
 output << "# ---------------------------------------------- brackets " << endl;
 output << "# 10,30,50,70,90 => 20% brackets    110 => top 1% bracket " << endl; 
 output << "# 10=>  0-20%  <taxes paid>  % of the pie "                 << endl; 
 output << "# 30=> 20-40%  <taxes paid>  % of the pie "                 << endl; 
 output << "# etc ... " << endl;
 output << "# ------------------------------------------ bracket data " << endl; 
 double dtemp = 1.0/(0.20*N_earners_);
 output << " 10 " << dtemp*sum_bin1 << "  " << p1 << endl; // between 0% to 20%
 output << " 30 " << dtemp*sum_bin2 << "  " << p2 << endl; //between 20% to 40%
 output << " 50 " << dtemp*sum_bin3 << "  " << p3 << endl; //between 40% to 60%
 output << " 70 " << dtemp*sum_bin4 << "  " << p4 << endl; //between 60% to 80%
 output << " 90 " << dtemp*sum_bin5 << "  " << p5 << endl; //between 80 to 100%
 dtemp = 1.0/(0.01*N_earners_);
 output << "110 " << dtemp*sum_bin6 << "  " << p6 << endl;         // => top 1%
 output.close();
};

 void TaxModel::market_income_brackets()
{
 std::string filename;
 fstream output;
// ---------------------------------------------------------- write diving lines
 double max_earnings_distr = -1.0e20; 
 double min_earnings_distr = 1.0e50; 
    for(int i=0; i<N_Esupport_; i++)
    {
       if( earnings_distribution_.at(i) > max_earnings_distr )
       {
       max_earnings_distr = earnings_distribution_.at(i); 
       }
       if( earnings_distribution_.at(i) < min_earnings_distr )
       {
       min_earnings_distr = earnings_distribution_.at(i); 
       }
    }
// ----------------------------- set maximum at 1.25 greater than actual maximum
 max_earnings_distr *= 1.25;
// ------------------------------- set minimum at 1.25 lower than actual minimum
 min_earnings_distr /= 1.25;
// -------------------------------------------------------- dividing line at 20%
 filename = name_ + info_ + "_Pm20.inc";
 output.open( filename.c_str() , fstream::out );
 output << E20_earnings_ << "  " << min_earnings_distr << endl;
 output << E20_earnings_ << "  " << max_earnings_distr << endl;
 output.close();
// -------------------------------------------------------- dividing line at 40%
 filename = name_ + info_ + "_Pm40.inc";
 output.open( filename.c_str() , fstream::out );
 output << E40_earnings_ << "  " << min_earnings_distr << endl;
 output << E40_earnings_ << "  " << max_earnings_distr << endl;
 output.close();
// -------------------------------------------------------- dividing line at 50%
 filename = name_ + info_ + "_Pm50.inc";
 output.open( filename.c_str() , fstream::out );
 output << E50_earnings_ << "  " << min_earnings_distr << endl;
 output << E50_earnings_ << "  " << max_earnings_distr << endl;
 output.close();
// -------------------------------------------------------- dividing line at 60%
 filename = name_ + info_ + "_Pm60.inc";
 output.open( filename.c_str() , fstream::out );
 output << E60_earnings_ << "  " << min_earnings_distr << endl;
 output << E60_earnings_ << "  " << max_earnings_distr << endl;
 output.close();
// -------------------------------------------------------- dividing line at 80%
 filename = name_ + info_ + "_Pm80.inc";
 output.open( filename.c_str() , fstream::out );
 output << E80_earnings_ << "  " << min_earnings_distr << endl;
 output << E80_earnings_ << "  " << max_earnings_distr << endl;
 output.close();
// -------------------------------------------------------- dividing line at 99%
 filename = name_ + info_ + "_Pm99.inc";
 output.open( filename.c_str() , fstream::out );
 output << E99_earnings_ << "  " << min_earnings_distr << endl;
 output << E99_earnings_ << "  " << max_earnings_distr << endl;
 output.close();
// -------------------------------------------------------- dividing line at ave
 filename = name_ + info_ + "_Pmav.inc";
 output.open( filename.c_str() , fstream::out );
 output << ave_earnings_ << "  " << min_earnings_distr << endl;
 output << ave_earnings_ << "  " << max_earnings_distr << endl;
 output.close();
// ----------------------------------------------------------- setup file output
 filename = name_ + info_ + "_miQ.inc";
 output.open( filename.c_str() , fstream::out );
 output << "# tot_earnings = " << tot_earnings_ << endl;
 output << "# max_earnings = " << max_earnings_ << endl;
 output << "# ave_earnings = " << ave_earnings_ << endl;
 output << "# E20_earnings = " << E20_earnings_ << endl;
 output << "# E40_earnings = " << E40_earnings_ << endl;
 output << "# E50_earnings = " << E50_earnings_ << endl;
 output << "# E60_earnings = " << E60_earnings_ << endl;
 output << "# E80_earnings = " << E80_earnings_ << endl;
 output << "# E99_earnings = " << E99_earnings_ << endl;
 output << "# std_earnings = " << std_earnings_ << endl;
 output << "# min_earnings = " << min_earnings_ << endl;
 output << "# num_earners  = " << N_earners_    << endl;
 output << "# ------------------------------- " << endl;
// -------------------------------------------------------------------- brackets
 double sum_bin1 = 0.0;
 double sum_bin2 = 0.0;
 double sum_bin3 = 0.0;
 double sum_bin4 = 0.0;
 double sum_bin5 = 0.0;
 double sum_bin6 = 0.0;
    for(int i=0; i<percentile_variable_.size(); i++)
    {
    if( percentile_variable_.at(i) < 0.0 ) break;
       if( percentile_variable_.at(i) < 20.0 ) 
       {
       sum_bin1 += rank_ordered_earnings_.at(i)*d_percentile_variable_.at(i); 
       }
       else if( percentile_variable_.at(i) < 40.0 ) 
       {
       sum_bin2 += rank_ordered_earnings_.at(i)*d_percentile_variable_.at(i); 
       }
       else if( percentile_variable_.at(i) < 60.0 ) 
       {
       sum_bin3 += rank_ordered_earnings_.at(i)*d_percentile_variable_.at(i); 
       }
       else if( percentile_variable_.at(i) < 80.0 ) 
       {
       sum_bin4 += rank_ordered_earnings_.at(i)*d_percentile_variable_.at(i); 
       }
       else
       {
       sum_bin5 += rank_ordered_earnings_.at(i)*d_percentile_variable_.at(i); 
       }
       if( percentile_variable_.at(i) >= 99.0 )
       {
       sum_bin6 += rank_ordered_earnings_.at(i)*d_percentile_variable_.at(i); 
       }
    }
// ---------------------------------------------------------------------- output
 double sum = 0.0;
 sum += sum_bin1;
 sum += sum_bin2;
 sum += sum_bin3;
 sum += sum_bin4;
 sum += sum_bin5;
 output << "# total_earnings = " << sum << endl;
 double p1 = 100.0*sum_bin1/sum; 
 double p2 = 100.0*sum_bin2/sum; 
 double p3 = 100.0*sum_bin3/sum; 
 double p4 = 100.0*sum_bin4/sum; 
 double p5 = 100.0*sum_bin5/sum; 
 double p6 = 100.0*sum_bin6/sum; 
 output << "# ---------------------------------------------- brackets " << endl;
 output << "# 10,30,50,70,90 => 20% brackets    110 => top 1% bracket " << endl; 
 output << "# 10=>  0-20%  <market income>  % of the pie "              << endl; 
 output << "# 30=> 20-40%  <market income>  % of the pie "              << endl; 
 output << "# etc ... " << endl;
 output << "# ------------------------------------------ bracket data " << endl; 
 double dtemp = 1.0/(0.20*N_earners_);
 output << " 10 " << dtemp*sum_bin1 << "  " << p1 << endl; // between 0% to 20%
 output << " 30 " << dtemp*sum_bin2 << "  " << p2 << endl; //between 20% to 40%
 output << " 50 " << dtemp*sum_bin3 << "  " << p3 << endl; //between 40% to 60%
 output << " 70 " << dtemp*sum_bin4 << "  " << p4 << endl; //between 60% to 80%
 output << " 90 " << dtemp*sum_bin5 << "  " << p5 << endl; //between 80 to 100%
 dtemp = 1.0/(0.01*N_earners_);
 output << "110 " << dtemp*sum_bin6 << "  " << p6 << endl;         // => top 1%
 output.close();
};

void TaxModel::set_earnings_support() 
{
// -------------------------------------------------------------- record density
// steps:    10000      39000       86000       96000       286000        586000
// Nsize = 10000x1 + 29000x10 + 47000x100 + 10000x500 +190000x1000 +300000x10000 
// earnings: 10000     300000     5000000    10000000    200000000    5000000000
   int Nsize = 586001;
   earnings_variable_.reserve(Nsize); 
   earnings_variable_.resize(Nsize); 
   d_earnings_variable_.reserve(Nsize); 
   d_earnings_variable_.resize(Nsize); 
   earnings_distribution_.reserve(Nsize);  
   earnings_distribution_.resize(Nsize);  
   percentile_variable_.reserve(Nsize);  
   percentile_variable_.resize(Nsize);  
   d_percentile_variable_.reserve(Nsize);  
   d_percentile_variable_.resize(Nsize);  
   rank_ordered_earnings_.reserve(Nsize); 
   rank_ordered_earnings_.resize(Nsize); 
   transfer_density_.reserve(Nsize); 
   transfer_density_.resize(Nsize); 
   deduction_density_.reserve(Nsize);  
   deduction_density_.resize(Nsize);  
   cptl_loss_density_.reserve(Nsize);
   cptl_loss_density_.resize(Nsize);
   receipt_T_density_.reserve(Nsize); 
   receipt_T_density_.resize(Nsize); 
   receipt_D_density_.reserve(Nsize); 
   receipt_D_density_.resize(Nsize); 
   receipt_I_density_.reserve(Nsize); 
   receipt_I_density_.resize(Nsize); 
   least_tax_density_.reserve(Nsize); 
   least_tax_density_.resize(Nsize); 
   tax_rate_.reserve(Nsize);                       
   tax_rate_.resize(Nsize); 
// ---------------------------------------------------------------- build support
   double E = 0.0;                                                 // start range
// ----------------------------------------------------------------------- dE = 1
   double dE = 1.0; 
   double dE2 = 0.5*dE;
   E += dE2;
      for(int i=0; i<10000; i++)
      {
      earnings_variable_.at(i) = E;  
      d_earnings_variable_.at(i) = dE;  
//    cout << i << "  " 
//         << earnings_variable_.at(i) << "  " 
//         << d_earnings_variable_.at(i) << endl;
      E += dE; 
      }
   E -= dE2;                                                      // finish range
// cout << "range 1 ending = " << E << endl;
// ---------------------------------------------------------------------- dE = 10
   dE = 10.0; 
   dE2 = 0.5*dE; 
   E += dE2;
      for(int i=10000; i<39000; i++)
      {
      earnings_variable_.at(i) = E;  
      d_earnings_variable_.at(i) = dE;  
//    cout << i << "  " 
//         << earnings_variable_.at(i) << "  " 
//         << d_earnings_variable_.at(i) << endl;
      E += dE; 
      }
   E -= dE2;                                                      // finish range
// cout << "range 2 ending = " << E << endl;
// --------------------------------------------------------------------- dE = 100
   dE = 100.0; 
   dE2 = 0.5*dE; 
   E += dE2;
      for(int i=39000; i<86000; i++)
      {
      earnings_variable_.at(i) = E;  
      d_earnings_variable_.at(i) = dE;  
//    cout << i << "  " 
//         << earnings_variable_.at(i) << "  " 
//         << d_earnings_variable_.at(i) << endl;
      E += dE; 
      }
   E -= dE2;                                                      // finish range
// cout << "range 3 ending = " << E << endl;
// --------------------------------------------------------------------- dE = 500
   dE = 500.0; 
   dE2 = 0.5*dE; 
   E += dE2;
      for(int i=86000; i<96000; i++)
      {
      earnings_variable_.at(i) = E;  
      d_earnings_variable_.at(i) = dE;  
//    cout << i << "  " 
//         << earnings_variable_.at(i) << "  " 
//         << d_earnings_variable_.at(i) << endl;
      E += dE; 
      }
   E -= dE2;                                                      // finish range
// cout << "range 4 ending = " << E << endl;
// -------------------------------------------------------------------- dE = 1000
   dE = 1000.0; 
   dE2 = 0.5*dE; 
   E += dE2;
      for(int i=96000; i<286000; i++)
      {
      earnings_variable_.at(i) = E;  
      d_earnings_variable_.at(i) = dE;  
//    cout << i << "  " 
//         << earnings_variable_.at(i) << "  " 
//         << d_earnings_variable_.at(i) << endl;
      E += dE; 
      }
   E -= dE2;                                                      // finish range
// cout << "range 5 ending = " << E << endl;
// ------------------------------------------------------------------- dE = 10000
   dE = 1000.0; 
   dE2 = 0.5*dE; 
   E += dE2;
      for(int i=286000; i<586000; i++)
      {
      earnings_variable_.at(i) = E;  
      d_earnings_variable_.at(i) = dE;  
//    cout << i << "  " 
//         << earnings_variable_.at(i) << "  " 
//         << d_earnings_variable_.at(i) << endl;
      E += dE; 
      }
   E -= dE2;                                                      // finish range
// cout << "range 6 ending = " << E << endl;
};

void TaxModel::Exx_earnings()
{
  double dtemp;
  double delta20; 
  double delta40; 
  double delta50; 
  double delta60; 
  double delta80; 
  double delta99; 
  delta20 = 1.0e40;
  delta40 = 1.0e40;
  delta50 = 1.0e40;
  delta60 = 1.0e40;
  delta80 = 1.0e40;
  delta99 = 1.0e40;
     for(int i=0; i<percentile_variable_.size(); i++)
     {
     if( percentile_variable_.at(i) < 0.0 ) break;
// ----------------------------------------------------------- population at 20%
     dtemp = fabs(percentile_variable_.at(i) - 20.0);
        if( dtemp < delta20 )
        {
        delta20 = dtemp;
        E20_earnings_ = 100.0*rank_ordered_earnings_.at(i)/N_earners_;
        }
// ----------------------------------------------------------- population at 40%
     dtemp = fabs(percentile_variable_.at(i) - 40.0);
        if( dtemp < delta40 )
        {
        delta40 = dtemp;
        E40_earnings_ = 100.0*rank_ordered_earnings_.at(i)/N_earners_;
        }
// ----------------------------------------------------------- population at 50%
     dtemp = fabs(percentile_variable_.at(i) - 50.0);
        if( dtemp < delta50 )
        {
        delta50 = dtemp;
        E50_earnings_ = 100.0*rank_ordered_earnings_.at(i)/N_earners_;
        }
// ----------------------------------------------------------- population at 60%
     dtemp = fabs(percentile_variable_.at(i) - 60.0);
        if( dtemp < delta60 )
        {
        delta60 = dtemp;
        E60_earnings_ = 100.0*rank_ordered_earnings_.at(i)/N_earners_;
        }
// ----------------------------------------------------------- population at 80%
     dtemp = fabs(percentile_variable_.at(i) - 80.0);
        if( dtemp < delta80 )
        {
        delta80 = dtemp;
        E80_earnings_ = 100.0*rank_ordered_earnings_.at(i)/N_earners_;
        }
// ----------------------------------------------------------- population at 99%
     dtemp = fabs(percentile_variable_.at(i) - 99.0);
        if( dtemp < delta99 )
        {
        delta99 = dtemp;
        E99_earnings_ = 100.0*rank_ordered_earnings_.at(i)/N_earners_;
        }
     }
};

void TaxModel::std_earnings()
{
 double dtemp,E,sum0,sum1,sum2;
 sum0 = 0;
 sum1 = 0;
 sum2 = 0;
      for(int i=0; i<N_Esupport_; i++)
      {
      E = earnings_variable_.at(i); 
      dtemp = earnings_distribution_.at(i)*d_earnings_variable_.at(i);
      sum0 += dtemp;
      sum1 += E*dtemp;
      sum2 += E*E*dtemp;
// ------------------------------------------------------------------ population 
      }
  double m1 = sum1/sum0;
  double m2 = sum2/sum0;
  dtemp = m2 - m1*m1; 
   if( dtemp < 0.01 ) 
   {
   std_earnings_ = 0.0; 
   }
   else
   {
   std_earnings_ = sqrt( m2 - m1*m1); 
   }
};

void TaxModel::renormalize() 
{
   double sum = 0.0;
      for(int i=0; i<N_Esupport_; i++)
      {
      sum += earnings_distribution_.at(i)*d_earnings_variable_.at(i);
      }
   double renormalize = N_earners_/sum; 
   double scale = 100.0/N_earners_;
   double E,dtemp;
   double p = 0.0;
// ---------------------------------------- initialize percentile_variable_.at(i)
      for(int i=N_Esupport_-10; i<percentile_variable_.size(); i++)
      {
      percentile_variable_.at(i) = -1.0;
      }
// --------------------------------------
      for(int i=0; i<N_Esupport_; i++)
      {
      E = earnings_variable_.at(i); 
      earnings_distribution_.at(i) *= renormalize;
// ------------------------------------------------------------------ population 
      dtemp = earnings_distribution_.at(i)*d_earnings_variable_.at(i);
      rank_ordered_earnings_.at(i) = E/scale;                  // total earnings
      d_percentile_variable_.at(i) = dtemp*scale;
      percentile_variable_.at(i) = p + 0.5*d_percentile_variable_.at(i);
      p += d_percentile_variable_.at(i);
         if( percentile_variable_.at(i) > 99.999999 ) 
         {
         percentile_variable_.at(i) = -1.0;
         N_Esupport_ = i;
         break; 
         }
         if( earnings_variable_.at(i) > 0.9999999*max_earnings_ ) 
         {
         percentile_variable_.at(i) = -1.0;
         N_Esupport_ = i;
         break; 
         }
      }
// ----------------------------------------------------------- fix normalization
   sum = 0.0; 
      for(int i=0; i<percentile_variable_.size(); i++)
      {
      if( percentile_variable_.at(i) < 0.0 ) break;
      sum += rank_ordered_earnings_.at(i)*d_percentile_variable_.at(i); 
      }
// ----------------------------------------------------------------- error check
      if( fabs( 1.0 - sum/tot_earnings_ ) > 0.001 )
      {
      cout << "ERROR: tot_earnings does not sum up correctly " << endl; 
      cout << "       %deviation = " << 100.0*(1.0 - sum/tot_earnings_) << endl;
      cout << endl; 
      }
      else
      {
      renormalize = tot_earnings_/sum; 
      }
// ------------------------------------------------------- correct normalization
      for(int i=0; i<percentile_variable_.size(); i++)
      {
      if( percentile_variable_.at(i) < 0.0 ) break;
      rank_ordered_earnings_.at(i) *= renormalize;
      }
};

void TaxModel::calc_Kfunction() 
{
// ------------------------------------------------- calculate beta(alpha,kappa)
   double i2k = 0.5/kappa_; 
   double i2a = 0.5/alpha_; 
   double t1 = 1.0/alpha_; 
   double g1 = gamma_function(t1); 
// ---------------------------------------------
   double t2 = (i2k - i2a); 
   double g2 = gamma_function(t2); 
// ---------------------------------------------
   double t3 = (i2k + i2a); 
   double g3 = gamma_function(t3); 
// ---------------------------------------------
   double factor1 = g1/(kappa_ + alpha_); 
   double factor2 = g2/g3; 
   beta_ = i2k* pow( factor1*factor2 , alpha_ ); 
// cout << "beta = " << beta_ << endl; 
// ------------------------------------------------ start calculation of density
   double dtemp;
   double z; 
   double B; 
   double ia = 1.0/alpha_;
   double A = pow( beta_ , -ia ); 
/* ----------------------------------------------------------- quantile function
      for(double u=0.0001; u<0.99999999; u+=0.0001)
      {
      z = 1.0 - u; 
      B = - lnK(z, kappa_); 
      dtemp = A*pow(B,ia);  
      cout << u << "  " << dtemp << endl; 
      }
----------------------------------------------------------------------------- */
   double u = 0.9999999;                      // good for N_earners > 10,000,000
   z = 1.0 - u; 
   B = - lnK(z, kappa_); 
   double x_max = A*pow(B,ia);  
   max_earnings_ = x_max*ave_earnings_;
   double x; 
   double x_alpha; 
   double ab = alpha_*beta_;  
   double bk = beta_*kappa_;  
   double bk2 = bk*bk; 
   double a,b,c,d;
   double norm = ab*N_earners_/ave_earnings_; 
// --------------------------------------------------------- cutoff distribution 
   N_Esupport_ = earnings_variable_.size() - 1; 
   double E;
      for(int i=0; i<N_Esupport_; i++)
      {
      E = earnings_variable_.at(i); 
      x = E/ave_earnings_;
         if( x > x_max ) 
         {
         N_Esupport_ = i; 
         break; 
         }
      x_alpha = pow( x , alpha_ ); 
      a = norm*x_alpha/x; 
      b = -beta_*x_alpha; 
      c = expK(b,kappa_);
      d = sqrt( 1.0 + bk2*x_alpha*x_alpha );
      earnings_distribution_.at(i) = a*c/d;
      }
   renormalize();
}

void TaxModel::read_tax_data()
{
// --------------------------------------------- read input data from file fopt_ 
   {
   ifstream input;
   input.open( fopt_.c_str() , fstream::in );
// ------------------------------------------------------- check if file opended
      if( input.fail() ) 
      {
      cout << "ERROR: File " << fopt_ << " not found! " << endl;
      exit(-1);
      }
// ---------------------------------------------- read file to the end, not line
   vector<std::string> blurb1;           // blurbs that come from parsing string
   vector<std::string> blurb2;           // blurbs that come from parsing string
   std::string line;
   std::string word;
   std::getline(input, line);                                        // 1st line
      {
      std::stringstream ss(line);
         while( ss >> word )
         {
         blurb1.push_back(word);
         }
      }
   std::getline(input, line);                                        // 2nd line
      {
      std::stringstream ss(line);
         while( ss >> word )
         {
         blurb2.push_back(word);
         }
      }
   N_earners_ = atof( blurb1.at(1).c_str() );
   ave_earnings_ = atof( blurb2.at(1).c_str() );
// cout << "number of earners = " << N_earners_ << endl; 
// cout << "ave market income = " << ave_earnings_ << endl; 
   double x,y;
   vector<double> target;                                             // targets
      while( std::getline(input, line) )
      {
         std::stringstream ss(line);
         if(ss >> x >> y )
         {
//       cout << x << "   " << y << endl;
         target.push_back(y); 
         }
//       else
//       {
//       cout << "Skip line: does not follow format of two columns " << endl;
//       }
      }
   input.close();
   target1_ = target.at(0);
   target2_ = target.at(1);
   target3_ = target.at(2);
   target4_ = target.at(3);
   target5_ = target.at(4);
   target6_ = target.at(5);
   }
/*
cout << endl;
cout << "target1 = " << target1_ << endl;
cout << "target2 = " << target2_ << endl;
cout << "target3 = " << target3_ << endl;
cout << "target4 = " << target4_ << endl;
cout << "target5 = " << target5_ << endl;
cout << "target6 = " << target6_ << endl;
cout << endl;
*/
} 

bool TaxModel::set_market_income(const int & argc, char **const argv )
{
/* ------------------------------------------- write out summary of calculations
      for(int j=0; j<argc; j++)
      {
      cout << argv[j] << " ";
      }
   cout << endl; 
*/
// --------------------------------------------------------------- simple checks
   int flag_bad = -1;                                  // nothing bad is assumed
   tot_earnings_ = ave_earnings_*N_earners_; 
      if( N_earners_ < 1.0 ) 
      {
      cout << "ERROR: number of earners is too small " << endl;
      cout << "       requested value = " << N_earners_ << endl;
      flag_bad = 1;
      }
      if( tot_earnings_ < 0.01 ) 
      {
      cout << "ERROR: total earnings is too small " << endl;
      cout << "       requested value = " << tot_earnings_ << endl;
      flag_bad = 1;
      }
      if( ave_earnings_ < 0.01 )  // on average: 1 penny per year is low income 
      {
      cout << "ERROR: average earnings is too small " << endl;
      cout << "       requested value = " << ave_earnings_ << endl;
      flag_bad = 1;
      }
      if( irs_revenue_ < 0.01 )     // can't be less than 1 penny 
      {
      cout << "ERROR: tax revenue is too small " << endl;
      cout << "       requested value = " << irs_revenue_ << endl;
      flag_bad = 1;
      }
      if( irs_transfr_ < 0.0 ) 
      {
      cout << "ERROR: tax revenue cannot be negative " << endl;
      cout << "       requested value = " << irs_transfr_ << endl;
      flag_bad = 1;
      }
      if( L_percntile_ < 0.0 ) 
      {
      cout << "ERROR: Basic deduction cannot be negative " << endl;
      cout << "       requested value = " << L_percntile_ << endl;
      flag_bad = 1;
      }
      if( L_percntile_ > 99.0 ) 
      {
      cout << "ERROR: maximum basic deduction % is 99.0% " << endl;
      cout << "       requested value = " << L_percntile_ << endl;
      flag_bad = 1;
      }
      if( H_percntile_ < 0.0 ) 
      {
      cout << "ERROR: H_percntile_ cannot be negative " << endl;
      cout << "       requested value = " << H_percntile_ << endl;
      flag_bad = 1;
      }
      if( H_percntile_ > 99.0 ) 
      {
      cout << "ERROR: maximum H_percntile_ is 99.0% " << endl;
      cout << "       requested value = " << H_percntile_ << endl;
      flag_bad = 1;
      }
      if( 100.0*CL_ <  -0.0000000001 ) 
      {
      cout << "ERROR: minimum capital loss relative to profit is 0%" << endl;
      cout << "       requested value = " << 100.0*CL_ << endl;
      flag_bad = 1;
      }
      if( 100.0*CL_ > 70.0000000001 ) 
      {
      cout << "ERROR: maximum capital loss relative to profit is 70%" << endl;
      cout << "       requested value = " << 100.0*CL_ << endl;
      flag_bad = 1;
      }
      if( inc_percent_ < 0.0 ) 
      {
      cout << "ERROR: inc_percent_ cannot be negative " << endl;
      cout << "       requested value = " << inc_percent_ << endl;
      flag_bad = 1;
      }
      if( inc_percent_ > 100.0 ) 
      {
      cout << "ERROR: inc_percent_ cannot be more than 100% " << endl;
      cout << "       requested value = " << inc_percent_ << endl;
      flag_bad = 1;
      }
      if( tot_earnings_ < irs_revenue_ ) 
      {
      cout << "ERROR: tax revenue cannot be more than total earnings " << endl;
      cout << "       requested total earnings = " << tot_earnings_ << endl;
      cout << "       requested tax revenue    = " << irs_revenue_ << endl;
      flag_bad = 1;
      }
      if( irs_transfr_ > irs_revenue_ ) 
      {
      cout << "ERROR: government transfer cannot be more than tax revenue " << endl;
      cout << "       requested gov. transfer  = " << irs_transfr_ << endl;
      cout << "       requested tax revenue    = " << irs_revenue_ << endl;
      flag_bad = 1;
      }
// ---------------------------------------- abort if parameters are out of range
   if( flag_bad > 0 ) return false;
   dP_ = 0.1;                                              // tenth of a percent
// ----------------------------------------------------- set up model parameters
   if( model_ == uniform )
   {
      if( sigma_ >= 0.0 ) 
      {
      cout << "ERROR: Cannot specify standard deviation " << endl;
      return false;
      }
   max_earnings_ = 2.0*ave_earnings_; 
   double dE = max_earnings_/100000;
   double density = tot_earnings_/dE;
   double dE2 = 0.5*dE;
// ------------------------------------------------------------------- 1st point
   earnings_variable_.push_back(0.0); 
   d_earnings_variable_.push_back(dE); 
   earnings_distribution_.push_back(0.0);  
// ------------------------------------------------------------------- 2nd point
   earnings_variable_.push_back(ave_earnings_ - dE2); 
   d_earnings_variable_.push_back(dE); 
   earnings_distribution_.push_back(0.0);  
// ------------------------------------------------------------------- 3rd point
   earnings_variable_.push_back(ave_earnings_); 
   d_earnings_variable_.push_back(dE); 
   earnings_distribution_.push_back(density);  
// ------------------------------------------------------------------- 4th point
   earnings_variable_.push_back(ave_earnings_ + dE2); 
   d_earnings_variable_.push_back(dE); 
   earnings_distribution_.push_back(0.0);  
// ------------------------------------------------------------------- 5th point
   earnings_variable_.push_back(max_earnings_); 
   d_earnings_variable_.push_back(dE); 
   earnings_distribution_.push_back(0.0);  
// ------------------------------------------------------------- set N_Esupport_
   N_Esupport_ = earnings_variable_.size(); 
// ------------------------------------------------------ population description
   percentile_variable_.reserve(1001);
   percentile_variable_.resize(1001);
   d_percentile_variable_.reserve(1001);
   d_percentile_variable_.resize(1001);
   rank_ordered_earnings_.reserve(1001);
   rank_ordered_earnings_.resize(1001);
   transfer_density_.reserve(1001); 
   transfer_density_.resize(1001); 
   deduction_density_.reserve(1001);  
   deduction_density_.resize(1001);  
   cptl_loss_density_.reserve(1001);
   cptl_loss_density_.resize(1001);
   receipt_T_density_.reserve(1001); 
   receipt_T_density_.resize(1001); 
   receipt_D_density_.reserve(1001); 
   receipt_D_density_.resize(1001); 
   receipt_I_density_.reserve(1001); 
   receipt_I_density_.resize(1001); 
   least_tax_density_.reserve(1001); 
   least_tax_density_.resize(1001); 
   tax_rate_.reserve(1001);    
   tax_rate_.resize(1001); 
// ------------------------------------------------------ population description
   density = tot_earnings_/100.0;              // 100 => accounts for percentage
   double p = 0.0;
      for(int i=0; i<percentile_variable_.size(); i++)
      {
      d_percentile_variable_.at(i) = dP_;                         // => uniform
      percentile_variable_.at(i) = p + 0.5*d_percentile_variable_.at(i);
      p += d_percentile_variable_.at(i);
      rank_ordered_earnings_.at(i) = density; 
      if( percentile_variable_.at(i) > 99.999999 ) percentile_variable_.at(i) = -1.0;
      }
   max_earnings_ = ave_earnings_; 
   min_earnings_ = ave_earnings_; 
   return true;
   }
   else if( model_ == linear )
   {
      if( sigma_ >= 0.0 ) 
      {
      cout << "ERROR: Cannot specify standard deviation " << endl;
      return false;
      }
   max_earnings_ = 3.0*ave_earnings_; 
   double slope = 2.0*N_earners_/(max_earnings_*max_earnings_); 
   double dE = max_earnings_/100000;
   double dE2 = 0.5*dE;
// -------------------------------------------------------------- record density
   earnings_variable_.reserve(100001); 
   earnings_variable_.resize(100001); 
   d_earnings_variable_.reserve(100001); 
   d_earnings_variable_.resize(100001); 
   earnings_distribution_.reserve(100001);  
   earnings_distribution_.resize(100001);  
   percentile_variable_.reserve(100001);  
   percentile_variable_.resize(100001);  
   d_percentile_variable_.reserve(100001);  
   d_percentile_variable_.resize(100001);  
   rank_ordered_earnings_.reserve(100001); 
   rank_ordered_earnings_.resize(100001); 
   transfer_density_.reserve(100001); 
   transfer_density_.resize(100001); 
   deduction_density_.reserve(100001);  
   deduction_density_.resize(100001);  
   cptl_loss_density_.reserve(100001);
   cptl_loss_density_.resize(100001);
   receipt_T_density_.reserve(100001); 
   receipt_T_density_.resize(100001); 
   receipt_D_density_.reserve(100001); 
   receipt_D_density_.resize(100001); 
   receipt_I_density_.reserve(100001); 
   receipt_I_density_.resize(100001); 
   least_tax_density_.reserve(100001); 
   least_tax_density_.resize(100001); 
   tax_rate_.reserve(100001);    
   tax_rate_.resize(100001); 
// ------------------------------------------------------------- set N_Esupport_
   N_Esupport_ = earnings_variable_.size(); 
   double E = dE2;
   double scale = 100.0/N_earners_;
   double dtemp;
   double p = 0.0;
      for(int i=0; i<N_Esupport_; i++)
      {
      earnings_variable_.at(i) = E; 
      d_earnings_variable_.at(i) = dE; 
      earnings_distribution_.at(i) = slope*(max_earnings_ - E);  
// ------------------------------------------------------------------ population 
      dtemp = earnings_distribution_.at(i)*d_earnings_variable_.at(i);
      rank_ordered_earnings_.at(i) = E/scale;                  // total earnings
      d_percentile_variable_.at(i) = dtemp*scale;
      percentile_variable_.at(i) = p + 0.5*d_percentile_variable_.at(i);
      p += d_percentile_variable_.at(i);
      E += d_earnings_variable_.at(i);
         if( percentile_variable_.at(i) > 99.999999 ) 
         {
         percentile_variable_.at(i) = -1.0;
         N_Esupport_ = i;
         break; 
         }
         if( earnings_variable_.at(i) > 0.99999999*max_earnings_ ) 
         {
         percentile_variable_.at(i) = -1.0;
         N_Esupport_ = i;
         break; 
         }
      }
   min_earnings_ = 0.0; 
   return true;
   }
   else if( model_ == quadratic )
   {
      if( sigma_ >= 0.0 ) 
      {
      cout << "ERROR: Cannot specify standard deviation " << endl;
      return false;
      }
   max_earnings_ = 4.0*ave_earnings_; 
   double A = 3.0*N_earners_/(64.0*ave_earnings_*ave_earnings_*ave_earnings_); 
   double dE = max_earnings_/100000;
   double dE2 = 0.5*dE;
// -------------------------------------------------------------- record density
   earnings_variable_.reserve(100001); 
   earnings_variable_.resize(100001); 
   d_earnings_variable_.reserve(100001); 
   d_earnings_variable_.resize(100001); 
   earnings_distribution_.reserve(100001);  
   earnings_distribution_.resize(100001);  
   percentile_variable_.reserve(100001);  
   percentile_variable_.resize(100001);  
   d_percentile_variable_.reserve(100001);  
   d_percentile_variable_.resize(100001);  
   rank_ordered_earnings_.reserve(100001); 
   rank_ordered_earnings_.resize(100001); 
   transfer_density_.reserve(100001); 
   transfer_density_.resize(100001); 
   deduction_density_.reserve(100001);  
   deduction_density_.resize(100001);  
   cptl_loss_density_.reserve(100001);
   cptl_loss_density_.resize(100001);
   receipt_T_density_.reserve(100001); 
   receipt_T_density_.resize(100001); 
   receipt_D_density_.reserve(100001); 
   receipt_D_density_.resize(100001); 
   receipt_I_density_.reserve(100001); 
   receipt_I_density_.resize(100001); 
   least_tax_density_.reserve(100001); 
   least_tax_density_.resize(100001); 
   tax_rate_.reserve(100001);    
   tax_rate_.resize(100001); 
// ------------------------------------------------------------- set N_Esupport_
   N_Esupport_ = earnings_variable_.size(); 
   double E = dE2;
   double scale = 100.0/N_earners_;
   double dtemp,x;
   double p = 0.0;
      for(int i=0; i<N_Esupport_; i++)
      {
      earnings_variable_.at(i) = E; 
      d_earnings_variable_.at(i) = dE; 
      x = (max_earnings_ - E);  
      earnings_distribution_.at(i) = A*x*x;
// ------------------------------------------------------------------ population 
      dtemp = earnings_distribution_.at(i)*d_earnings_variable_.at(i);
      rank_ordered_earnings_.at(i) = E/scale;                  // total earnings
      d_percentile_variable_.at(i) = dtemp*scale;
      percentile_variable_.at(i) = p + 0.5*d_percentile_variable_.at(i);
      p += d_percentile_variable_.at(i);
      E += d_earnings_variable_.at(i);
         if( percentile_variable_.at(i) > 99.999999 ) 
         {
         percentile_variable_.at(i) = -1.0;
         N_Esupport_ = i; 
         break; 
         }
         if( earnings_variable_.at(i) > 0.9999999*max_earnings_ ) 
         {
         percentile_variable_.at(i) = -1.0;
         N_Esupport_ = i; 
         break; 
         }
      }
   min_earnings_ = 0.0; 
   return true;
   }
   else if( model_ == log_normal)
   {
      if( sigma_ < 0.0 ) 
      {
      cout << "ERROR: Must specify standard deviation " << endl;
      return false;
      }
   double dtemp = sigma_/ave_earnings_;
   double Asig2 = log( 1.0 + dtemp*dtemp );                 // alternate sigma^2
   double A_sig = sqrt(Asig2); 
   double A_ave = log( ave_earnings_ ) - 0.5*Asig2;         // alternate average
// P(E) = ( 1.0/(E*sqrt(2*pi)*A_sig) )*exp( -( log(E) - A_ave )^2 /(2*Asig2) ) 
// -----------------------------------------------------------------------------
   dtemp = A_ave + 8.0*A_sig;           // = log(E_max) set 8 x A_sig from A_ave 
   max_earnings_ = exp(dtemp);                            // on linear scale now
// ----------------------- record density on non-uniform absolute earnings scale
   set_earnings_support();          // provides support for accurate integration


// ------------------------------------------------------------- set N_Esupport_
   N_Esupport_ = earnings_variable_.size() - 1; 
   double E;
   double scale = 100.0/N_earners_;
   double A = N_earners_/( A_sig*sqrt(2.0*3.141592654) );
   double B = 1.0/(2.0*Asig2); 
   double x;
   double p = 0.0;
      for(int i=0; i<N_Esupport_; i++)
      {
      E = earnings_variable_.at(i); 
//    cout << i << "   " << earnings_variable_.at(i)  << endl;   
//    set_earnings_support() explains jumps in density due to change in bin size 
         if( E > max_earnings_ )
         {
         N_Esupport_ = i;
         break; 
         }
      x = log(E) - A_ave;  
      earnings_distribution_.at(i) = (A/E)/exp(B*x*x);
// ------------------------------------------------------------------ population 
      dtemp = earnings_distribution_.at(i)*d_earnings_variable_.at(i);
      rank_ordered_earnings_.at(i) = E/scale;                  // total earnings
      d_percentile_variable_.at(i) = dtemp*scale;
//    cout << i << "   " << d_percentile_variable_.at(i) << endl;     
//    set_earnings_support() explains jumps in density due to change in bin size 
      percentile_variable_.at(i) = p + 0.5*d_percentile_variable_.at(i);
      p += d_percentile_variable_.at(i);
         if( percentile_variable_.at(i) > 99.999999 ) 
         {
         percentile_variable_.at(i) = -1.0;
         N_Esupport_ = i; 
         break; 
         }
         if( earnings_variable_.at(i) > 0.9999999*max_earnings_ ) 
         {
         percentile_variable_.at(i) = -1.0;
         N_Esupport_ = i; 
         break; 
         }
      }
   renormalize();
   min_earnings_ = 0.0; 
   return true;
   }
   else if( model_ == k_function )
   {
      if( alpha_ < 0.0 ) 
      {
      cout << "ERROR: Must specify alpha for K-function" << endl;
      return false;
      }
      if( kappa_ < 0.0 ) 
      {
      cout << "ERROR: Must specify kappa for K-function" << endl;
      return false;
      }
// ----------------------- record density on non-uniform absolute earnings scale
   set_earnings_support();          // provides support for accurate integration
// ---------------------------------------------- calculate Kfunction properties
   calc_Kfunction();
   min_earnings_ = 0.0; 
   return true;
   }
   else
   {
   cout << "ERROR: unknown model type was specified " << endl;
   return false;
   }
}

void TaxModel::initialize_endaid_line()
{
// --------------------------------------------------------------------- purpose
// REMARK 1:
// Because this program is really to test the victory tax system, the input 
// variables is either basic deduction (BDd) as a dollar amount or (BD) as a 
// percentile of the population that must be converted to a dollar amount. 
// After doing this, the end of aid line and the percentile quantities are 
// reconciled. That is, given one, the other is determined. 
// REMARK 2:
// Different tax models may have different end aid lines to keep the same 
// poverty level. End of aid line is not the same thing as where the poverty 
// line is, because it depends on factors such as deductables and whether the
// government transfer is taxed or not. 
// IMPORTANT:
// input_Eend_aid_  could be altered within the program holding all other 
// input variables the same, and, the entire calculation could be repeated 
// without any worries about initialization because everything is recalculated
// properly, provided the dependencies are respected.  
// --------------------------------------------- calculate end aid line earnings
    if( input_Eend_aid_ < 0.0 ) 
    {
    double delta = 1.0e40;
    double Eend_aid;                           // earnings defining end aid line
    double dtemp;
       for(int i=0; i<percentile_variable_.size(); i++)
       {
       if( percentile_variable_.at(i) < 0.0 ) break;
// ----------------------------------------------------------- population at -BD
       dtemp = fabs(percentile_variable_.at(i) - L_percntile_);
          if( dtemp < delta )
          {
          delta = dtemp;
          Eend_aid = 100.0*rank_ordered_earnings_.at(i)/N_earners_;
          }
       }
    Eend_aid_ = Eend_aid;
    }
    else
    {
    Eend_aid_ = input_Eend_aid_; 
    double delta = 1.0e40;
    double endaid_line = 0.0;         // local variable, assume 0 at first
    double dtemp;
    double scale = 100.0/N_earners_; 
       for(int i=0; i<percentile_variable_.size(); i++)
       {
       if( percentile_variable_.at(i) < 0.0 ) break;
// --------------------------------------------------------- dollar value at -BD
       dtemp = fabs( scale*rank_ordered_earnings_.at(i) - Eend_aid_);
          if( dtemp < delta )
          {
          delta = dtemp;
          endaid_line = percentile_variable_.at(i);
          }
       }
    L_percntile_ = endaid_line;
    }
// cout << endl;
// cout << "Eend_aid = " << Eend_aid_ << endl;
// cout << endl;
}

void TaxModel::initialize_tax_revenue() 
{
// NOTE DEPENDENCIES that must be done before running this subroutine.
// 1st must run initialize_endaid_line() to set the end of government aid line. 
// Begin with determining net_revenue_ and mia_transfr_ 
// Make estimates for deductions and write deduction information to all files
// --------------------------------------------------------------------- purpose
// This function defines the amount of taxes to collect and figures out the 
// amount of government transfer that needs to be collected to get up to the 
// end of aid line. It also records details of how the deductions work in the
// victory tax model and makes some models for itemized deductions and capital
// losses. This model for deductions is used as a representative description 
// to see how government assistance is distributed, which is in the form of 
// direct transfers and three forms of deductions.
// --------------------------------------------------------- set up output files
 std::string filename;
 fstream output;
 filename = name_ + info_ + "_inf.ded";        // information on deduction model
 output.open( filename.c_str() , fstream::out );
 output << "TaxModel::initialize_tax_revenue() is function in question" << endl;
 output << "   Calculates ... " << endl;
 output << "1. Taxes to be collected (model independent)" << endl;
 output << "2. Government transfer dispensed (model independent)" << endl;
 output << "3. Estimated deductions (partially model independent)" << endl;
 output << "   Capital loss deductions are model independent" << endl;
 output << "   Basic & itemized deductions are for victory & flat taxes"<< endl;
 output << "   Basic & itemized deductions are NOT for progressive tax" << endl;
 output << "---------------------------------------------------------- "<< endl;
 output << "   TD = MIN[ (BD + ID), MAX[BD,MD] , MP*AGI ] " << endl;
 output << "   TD = total tax deduction " << endl;
 output << "   BD = basic tax deduction " << endl;
 output << "   ID = itemized deduction " << endl;
 output << "   MD = maximum deduction " << endl;
 output << "   MP = maximum percent of adjusted gross income " << endl;
 output << "   IL = estimated capital losses from investments " << endl;
 output << "   q  = income percentile ranging from 0 to 1. " << endl;
 output << "   MI = market income, writen as E within the program" << endl;
 output << "   E  = market income based on population MI-distribution " << endl;
 output << "        accounts for earnings plus investment income " << endl;
 output << "        E=AGI = earnings + (capital gains - capital losses)"<< endl;
 output << "   CL = capital loss based on percentage of capital gains" << endl;
 output << "        on command line: CL ranges from 10 to 70 percent" << endl;
 output << "        within program CL --> CL_ ranging from 0.1 to 0.7 " << endl;
 output << "        Inet = Gains - Loss   CL_ = Loss/Gains " << endl;
 output << "        Inet ~ E*q^4 " << endl;
 output << "---------------------------------------------------------- "<< endl;
 output << "   ID = 0.5*(E - BD)*q^2" << endl;
 output << "   IL = BL_*E*q^4   where BL_ = CL_/(1-CL_) " << endl;
 output << "        BL_ = " << BL_ << endl;
 output << "        CL  = " << 100.0*CL_ << endl;
 output << "---------------------------------------------------------- "<< endl;
 fstream out_cld,out_evd,out_clr,out_tvt,out_dir,out_tvr;
 filename = name_ + info_ + "_cld.ded";       // capital loss deduction estimate
 out_cld.open( filename.c_str() , fstream::out );
 filename = name_ + info_ + "_edv.ded";        // victory tax deduction estimate
 out_evd.open( filename.c_str() , fstream::out );
 filename = name_ + info_ + "_tvt.ded";           // total VT deduction estimate
 out_tvt.open( filename.c_str() , fstream::out );
 filename = name_ + info_ + "_clr.ded";          // capital loss to income ratio
 out_clr.open( filename.c_str() , fstream::out );
 filename = name_ + info_ + "_dir.ded";             // deduction to income ratio
 out_dir.open( filename.c_str() , fstream::out );
 filename = name_ + info_ + "_tvr.ded";           // est. tot. VT dedction ratio
 out_tvr.open( filename.c_str() , fstream::out );
// ------------------------------------------------------- determine tax revenue
 net_revenue_ = (irs_revenue_ - irs_transfr_);           // => model independent
// ----------------------------------------------- determine government transfer
 double scale = 100.0/N_earners_; 
 mia_transfr_ = 0.0;                     // market income assistance by transfer
 double dE; 
 double E; 
    for(int i=0; i<percentile_variable_.size(); i++)
    {
    if( percentile_variable_.at(i) < 0.0 ) break;
    E = scale*rank_ordered_earnings_.at(i);
    dE = (Eend_aid_ - E) - 0.1;                               // assistance test
                        // ^^^-------> harder to collect $$ by substracting dime 
       if( dE > 0.0 )                                   // => below end aid line
       {
       transfer_density_.at(i) = dE/scale;   // => government transfer necessary
       }
       else                                             // => above end aid line
       {
       transfer_density_.at(i) = 0.0;       // => no transfer above end aid line
       }
    mia_transfr_ += transfer_density_.at(i)*d_percentile_variable_.at(i);
//  cout << percentile_variable_.at(i) << "  " 
//       << transfer_density_.at(i) << endl; 
    }
 tax_revenue_ = net_revenue_ + mia_transfr_; 
// ---------------------------------------------- finishing writing to info.ded
 output << "           tax_revenue = " << tax_revenue_ << endl;
 output << "   government transfer = " << mia_transfr_ << endl;
 output << "           net_revenue = " << net_revenue_ << endl;
 output << "---------------------------------------------------------- "<< endl;
 output.close();
// ------------------------------------------------------ define basic deduction
   BD_ = Eend_aid_; 
// -------------------------------------------- calculate maximum deduction (MD)
    if( input_max_deduction_ < 0.0 ) 
    {
    double delta = 1.0e40;
    double Emax_deduction;                                  // maximum deduction
    double dtemp;
       for(int i=0; i<percentile_variable_.size(); i++)
       {
       if( percentile_variable_.at(i) < 0.0 ) break;
// -------------------------------------------------- population at H_percntile_
       dtemp = fabs(percentile_variable_.at(i) - H_percntile_);
          if( dtemp < delta )
          {
          delta = dtemp;
          Emax_deduction = scale*rank_ordered_earnings_.at(i);
          }
       }
    MD_ = Emax_deduction;
    }
    else
    {
    MD_ = input_max_deduction_;
//  H_percntile_                                           next to be determined
    double delta = 1.0e40;
    double H_percntile = 0.0;                    // assume maximum deduction = 0
    double dtemp;
       for(int i=0; i<percentile_variable_.size(); i++)
       {
       if( percentile_variable_.at(i) < 0.0 ) break;
// -------------------------------------------------- population at H_percntile_
       dtemp = fabs(scale*rank_ordered_earnings_.at(i) - MD_);
          if( dtemp < delta )
          {
          delta = dtemp;
          H_percntile = percentile_variable_.at(i);
          }
       }
    H_percntile_ = H_percntile;
    }
// ----------------------------------------- possibly increase maximum deduction 
    if( BD_ > MD_ )
    {
    MD_ = BD_;                      // => MAX[BD,MD] --> MD for consistency sake
    } 
// ---------------------------------------------- calculate estimated deductions
//               nominal_itemized_deduction = 0.5*Earnings*(income_percentile)^2 
//                      nominal_capital_loss = CL*Earnings*(income_percentile)^4
  double E_density;
  double ID,requested_deduction,maxPE; 
  double sum = 0.0; 
  double x,x2,f,r; 
  f = 0.01*inc_percent_;
     for(int i=0; i<percentile_variable_.size(); i++)
     {
     if( percentile_variable_.at(i) < 0.0 ) break;
     E_density = rank_ordered_earnings_.at(i);
     E = scale*E_density;      // non government transfer earnings => deductable
     maxPE = f*E;                                        // MP percent of income
     x = percentile_variable_.at(i)/100.0;
     x2 = x*x;
        if( E < BD_ )
        {
        requested_deduction = E; 
        }
        else
        {
        ID = 0.5*(E - BD_)*x2;
        requested_deduction = BD_ + ID; 
           if( requested_deduction > MD_ )
           {
           requested_deduction = MD_;
           }
        }
        if( requested_deduction > maxPE )
        {
        requested_deduction = maxPE; 
        }
     r = requested_deduction/E;                       // ratio for this earnings
     deduction_density_.at(i) = r*E_density;    // E_density has same ratio as E
     cptl_loss_density_.at(i) = BL_*E_density*x2*x2;
     }
// ----------------------------------------------------- prepare data for output
  vector<double> xp;
  vector<double> ycld;
  vector<double> yevd;
  vector<double> yclr;
  vector<double> ydir;
  double dx_target; 
  double x_target = -0.0001; 
  double x_deviation; 
     for(int i=0; i<percentile_variable_.size(); i++)
     {
     if( percentile_variable_.at(i) < 0.0 ) break;
     x_deviation = x_target - percentile_variable_.at(i); 
       if( x_deviation < 0.0 ) 
       {
       xp.push_back(  percentile_variable_.at(i) );
       r = scale*cptl_loss_density_.at(i);
       ycld.push_back(r);
       r = scale*deduction_density_.at(i);
       yevd.push_back(r);
       E_density = rank_ordered_earnings_.at(i) + transfer_density_.at(i);
       r = cptl_loss_density_.at(i)/E_density; 
       yclr.push_back(r);
       r = deduction_density_.at(i)/E_density; 
       ydir.push_back(r);
       update(x_target); 
          if( x_target < percentile_variable_.at(i) ) 
          {
          x_target = percentile_variable_.at(i) + 0.00001; 
          }
       } 
     }
// -------------------------------------------------------- write subset of data
    for(int i=0; i<xp.size(); i++)
    {
    out_cld << xp.at(i) << "  " << ycld.at(i) << endl;
    out_evd << xp.at(i) << "  " << yevd.at(i) << endl;
    out_tvt << xp.at(i) << "  " << ( yevd.at(i) + ycld.at(i) ) << endl;
    out_clr << xp.at(i) << "  " << yclr.at(i) << endl;
    out_dir << xp.at(i) << "  " << ydir.at(i) << endl;
    out_tvr << xp.at(i) << "  " << ( ydir.at(i) + yclr.at(i) ) << endl;
    }
 out_cld.close();
 out_evd.close();
 out_tvt.close();
 out_clr.close();
 out_dir.close();
 out_tvr.close();
}

void TaxModel::calc_prog_tax_revenues()
{
// NOTE DEPENDENCIES that must be done before running this subroutine.
// 1st must run initialize_endaid_line() to set the end of government aid line. 
// 2nd must run initialize_tax_revenue() to set mia_transfr_ 
// running other tax models before this one does not matter. 
// Begin with perfect linear progressive tax model revenue calculations
// ---------------------------------------------------------- set up output file
 std::string filename;
 fstream output;
 filename = name_ + info_ + "_pti.tax";           // progressive tax information
 output.open( filename.c_str() , fstream::out );
// ------------------------------- tax information on input reference tax system
 double dtemp = (irs_revenue_ - irs_transfr_);  
 output << "----------------------------------------------- REFERENCE " << endl; 
 output << "nominal tax collected = " << irs_revenue_ << endl; 
 output << "  government transfer = " << irs_transfr_ << endl;
 output << "    net tax collected = " << dtemp << endl;
// ----------------------------------------------------------- taxable transfers
 output << "----------------------------------------------- Government transfer" 
        << endl; 
   if( ON_ > 0.5 ) 
   {
   output << " tax income transfers: YES " << endl;
   }
   else
   {
   output << " tax income transfers: NO  " << endl;
   }
 output << " end aid line percentile = " << L_percntile_ << endl;
 output << "             end aid AGI = " << Eend_aid_ << endl;
// --------------------------- tax information on perfect progressive tax system
 output << "----------------------------------- Progressive tax model " << endl; 
 output << "nominal tax collected = " << tax_revenue_ << endl;
 output << "  government transfer = " << mia_transfr_ << endl;
 output << "    net tax collected = " << net_revenue_ << endl;
 output << "--------------------------------------------------------- " << endl;
// ---------------------------------------------- determine progressive tax rate
   Emin_deduction_ = 0;             // => change to a generic name for later use
   double taxable_income; 
   double effective_tax_rate;
   double taxable_income_density; 
// ----------------------------------------------------- calculate total revenue
   double total = 0.0; 
      for(int i=0; i<percentile_variable_.size(); i++)
      {
      if( percentile_variable_.at(i) < 0.0 ) break;
      taxable_income_density = rank_ordered_earnings_.at(i) 
                             + ON_*transfer_density_.at(i); 
      taxable_income = taxable_income_density*d_percentile_variable_.at(i); 
      effective_tax_rate = 0.01*(percentile_variable_.at(i)); //set max rate = 1
      total += effective_tax_rate*taxable_income;
      }
   progressive_tax_rate_ = tax_revenue_/total;
// ------------------------------------------------------- calculate tax_density
      for(int i=0; i<percentile_variable_.size(); i++)
      {
      if( percentile_variable_.at(i) < 0.0 ) break;
      taxable_income_density = rank_ordered_earnings_.at(i) 
                             + ON_*transfer_density_.at(i); 
      effective_tax_rate = 0.01*(percentile_variable_.at(i)*progressive_tax_rate_);
      least_tax_density_.at(i) = effective_tax_rate*taxable_income_density; 
      tax_rate_.at(i) = effective_tax_rate;              // record for later use
      }
 output << "        maximum tax rate = " << 100.0*progressive_tax_rate_ << endl;
 double ave_etr;
 P_etr_id_  = "_Ppt.tax";    // file ending for probability for average tax rate
 C_etr_id_  = "_Cpt.tax";   // file ending for cumulative prob. for ave tax rate
 ave_etr = calc_ave_etr();
 output << "             average ETR = " << ave_etr << endl;
// ----------------------------------------------------- determine poverty level
  double scale = 100.0/N_earners_; 
  double poverty_level = 1.0e40;
  double after_tax_income; 
  double tax_paid;
  double Eadd;
  double E;
      for(int i=0; i<percentile_variable_.size(); i++)
      {
      if( percentile_variable_.at(i) < 0.0 ) break;
      E = scale*rank_ordered_earnings_.at(i);
      Eadd = scale*transfer_density_.at(i);
      tax_paid = scale*least_tax_density_.at(i);     // only one case: MIN = MAX
      after_tax_income = E + Eadd - tax_paid; 
         if( after_tax_income < poverty_level )
         {
         poverty_level = after_tax_income; 
         }
      }
 poverty_level_ = poverty_level;
 output << "           poverty level = " << poverty_level << endl;
// --------------- estimate total government subsidy = (assistance + deductions)
  double r;
  double sum = 0.0; 
  double subsidy_density; 
     for(int i=0; i<percentile_variable_.size(); i++)
     {
     if( percentile_variable_.at(i) < 0.0 ) break;
     r = tax_rate_.at(i);         // ||----------------> = 0 when off, 1 when on
     receipt_T_density_.at(i) = (1 - ON_*r)*transfer_density_.at(i); 
     receipt_D_density_.at(i) = 0;    // no basic or itemized deductions allowed
     receipt_I_density_.at(i) = r*cptl_loss_density_.at(i);          // estimate
     subsidy_density = receipt_T_density_.at(i) 
                     + receipt_I_density_.at(i); 
//                   + receipt_D_density_.at(i)   => excluded in prog. tax model
     sum += subsidy_density*d_percentile_variable_.at(i); 
     }
   net_subsidy_ = sum; 
 output << "       estimated subsidy = " << net_subsidy_ << endl;
 output.close();
// -------------------------------------------------------- set file information
// -------------------------------------------------------------------- tax info
 ef_tax_id_ = "_let.tax";                           // linear effective tax rate
 lz_tax_id_ = "_pLz.tax";  // tax Lorenz curve for linear progressive tax system
 Q6_tax_id_ = "_ptQ.tax";        // % tax within 5-quintiles and top 1% quantile
 burden_id_ = "_ptp.tax";         // taxes paid in linear progressive tax system
// -------------------------------------------------------------- after tax info
 mx_ati_id_ = "_lpt.ati";     // linear-progressive after tax income (MIN = MAX)
 mn_ati_id_ = "xxxxxxxx";                   // <= does not exist since MIN = MAX
// -------------------------------------------------- government assistance info
 ratio_id_  = "_psr.gva";         // subsidy to income ratio for progressive tax
 kudos_id_  = "_pgt.gva";             // government transfer for progressive tax
 mingt_id_  = "xxxxxxxx";      // would be  vga.gva  but this is same as pgt.gva    
 gt_lz_id_  = "_pLz.gva";            // subsidy Lorenz curve for progressive tax
 Q6_gt_id_  = "_ptQ.gva";   // % assistance within 5-quintiles & top 1% quantile
}

void TaxModel::calc_victoryT_revenues()
{
// NOTE DEPENDENCIES that must be done before running this subroutine.
// 1st must run initialize_endaid_line() to set the end of government aid line. 
// 2nd must run initialize_tax_revenue() to set mia_transfr_ 
// running other tax models before this one does not matter. 
// Begin with victory tax model revenue calculations
// ------------------------------------- set up output file for victory tax rate
 std::string filename;
 fstream output;
 filename = name_ + info_ + "_vti.tax";               // victory tax information
 output.open( filename.c_str() , fstream::out );
// ------------------------------- tax information on input reference tax system
 double dtemp = (irs_revenue_ - irs_transfr_);  
 output << "----------------------------------------------- REFERENCE " << endl; 
 output << "nominal tax collected = " << irs_revenue_ << endl; 
 output << "  government transfer = " << irs_transfr_ << endl;
 output << "    net tax collected = " << dtemp << endl;
// ----------------------------------------------------------- taxable transfers
 output << "----------------------------------------------- Government transfer" 
        << endl; 
   if( ON_ > 0.5 ) 
   {
   output << " tax income transfers: YES " << endl;
   }
   else
   {
   output << " tax income transfers: NO  " << endl;
   }
 output << " end aid line percentile = " << L_percntile_ << endl;
 output << "             end aid AGI = " << Eend_aid_ << endl;
// --------------------------------------- tax information on victory tax system
 output << "----------------------------------------------- Victory tax model " 
        << endl; 
 output << "nominal tax collected = " << tax_revenue_ << endl;
 output << "  government transfer = " << mia_transfr_ << endl;
 output << "    net tax collected = " << net_revenue_ << endl;
 output << "                   BD = " << BD_ << endl;
 output << "      percentile @ BD = " << L_percntile_ << endl;
 output << "                   MD = " << MD_ << endl;
 output << "      percentile @ MD = " << H_percntile_ << endl;
 output << "                   MP = " << inc_percent_ << endl;
 output << "----------------------------------------------- " << endl;
// -------------------------------------------------- determine uniform tax rate
 Emin_deduction_ = BD_;                           // => change to a generic name
// -------------------------------------------- apply maximum deduction possible
 double scale = 100.0/N_earners_; 
 double requested_deduction; 
 double Etaxable;                                  // earnings that can be taxed
 double f,E; 
 f = 0.01*inc_percent_;
    for(int i=0; i<percentile_variable_.size(); i++)
    {
    if( percentile_variable_.at(i) < 0.0 ) break;
    E = scale*rank_ordered_earnings_.at(i);
    requested_deduction = f*E;                // => the largest request possible
       if( requested_deduction > MD_ )
       {
       requested_deduction = MD_;
       }
    Etaxable = (E - requested_deduction);               // taxable market income
    if( Etaxable < 0.0 ) Etaxable = 0.0; 
// -------------------------------------------- assume effective tax rate = 100%
    least_tax_density_.at(i) = Etaxable/scale       // pay tax on taxable income
                             + ON_*transfer_density_.at(i);      // ON_ = 1 or 0
                           //  ^^^=1 ============> tax must be paid on transfers
                           //  ON_ = 0 is possible with the -OFF flag
    }
// -------------------------------------------------- determine victory tax rate
   double taxable_income; 
// ----------------------------------------------------- calculate total revenue
   double total = 0.0; 
      for(int i=0; i<percentile_variable_.size(); i++)
      {
      if( percentile_variable_.at(i) < 0.0 ) break;
      taxable_income = least_tax_density_.at(i)*d_percentile_variable_.at(i); 
      total += taxable_income;                // => assumes same_tax_rate = 100%
      }
 same_tax_rate_ = tax_revenue_/total;
 double poverty_level = Eend_aid_*(1 - same_tax_rate_); 
 poverty_level_ = poverty_level;
// ---------------------------------------------------------- record information
     for(int i=0; i<percentile_variable_.size(); i++)
     {
     if( percentile_variable_.at(i) < 0.0 ) break;
     least_tax_density_.at(i) *= same_tax_rate_; 
     tax_rate_.at(i) = same_tax_rate_; 
     }
// ---------------------------------------- calculate average effective tax rate
 double ave_etr;
 P_etr_id_  = "_Pvt.tax";    // file ending for probability for average tax rate
 C_etr_id_  = "_Cvt.tax";   // file ending for cumulative prob. for ave tax rate
 ave_etr = calc_ave_etr();
// --------------- estimate total government subsidy = (assistance + deductions)
  double sum = 0.0; 
  double subsidy_density; 
  double r;
  r = same_tax_rate_; 
     for(int i=0; i<percentile_variable_.size(); i++)
     {
     if( percentile_variable_.at(i) < 0.0 ) break;
                                  // ||----------------> = 0 when off, 1 when on
     receipt_T_density_.at(i) = (1 - ON_*r)*transfer_density_.at(i); 
     receipt_D_density_.at(i) = r*deduction_density_.at(i);
     receipt_I_density_.at(i) = r*cptl_loss_density_.at(i);          // estimate
     subsidy_density = receipt_T_density_.at(i) 
                     + receipt_D_density_.at(i)
                     + receipt_I_density_.at(i); 
     sum += subsidy_density*d_percentile_variable_.at(i); 
     }
  net_subsidy_ = sum; 
// ------------------------------------------------------- write out information
 output << "        victory tax rate = " << 100.0*same_tax_rate_ << endl;
 output << "             average ETR = " << ave_etr << endl;
 output << "           poverty level = " << poverty_level << endl;
 output << "       estimated subsidy = " << net_subsidy_ << endl;
 output.close();
// -------------------------------------------------------- set file information
// -------------------------------------------------------------------- tax info
 ef_tax_id_ = "_etr.tax";            // effective tax rate in victory tax system
 lz_tax_id_ = "_vLz.tax";             // tax Lorenz curve for victory tax system
 Q6_tax_id_ = "_vtQ.tax";   // % tax quintiles & top 1% quantile for victory tax
 burden_id_ = "_vtp.tax";                    // taxes paid in victory tax system
// -------------------------------------------------------------- after tax info
 mx_ati_id_ = "_vtf.ati";        // max after tax income (FULL deductions taken)
 mn_ati_id_ = "_vts.ati";     // min after tax income (only STANDARD deductions)
// -------------------------------------------------- government assistance info
 ratio_id_  = "_vsr.gva";       // subsidy to income ratio in victory tax system
 kudos_id_  = "_vgt.gva";      // government transfer only in victory tax system
 mingt_id_  = "_vga.gva";     // government assistance via transfer & deductions
 gt_lz_id_  = "_vLz.gva";         // subsidy Lorenz curve for victory tax system
 Q6_gt_id_  = "_vtQ.gva";   // % assistance within 5-quintiles & top 1% quantile
}

void TaxModel::out_gov_subsidie_ratio()
{
// NOTE DEPENDENCIES that must be done before running this subroutine.
// 1st must run initialize_endaid_line() to set the end of government aid line. 
// 2nd must run initialize_tax_revenue() to set mia_transfr_ 
// 3rd must run tax model
// ---------------------------------------------------------------------- remark
// Keep in mind that subsidy calculations require certain assumptions about the
// the amount of itemized deductions and capital loss deductions. These are 
// representative estimates to provide useful insight into the characteristics 
// of the tax model. The assumptions for the deductions can be modified within
// initialize_tax_revenue()
//
// IMPORTANT:
// minimum after tax income is independent of deduction model 
// maximum after tax income is independent of deduction model
// extremes in after tax income depend only on the tax model. 
// --------------------------------------------- output government subsidy ratio
 std::string filename;
 fstream output,out_ati,out_mat;
 filename = name_ + info_ + ratio_id_;
 output.open( filename.c_str() , fstream::out );
 filename = name_ + info_ + mx_ati_id_; 
 out_ati.open( filename.c_str() , fstream::out );
 double scale = 100.0/N_earners_; 
 double min_after_tax_income;
 double max_after_tax_income;
 double pre_tax_income;
 double subsidie_ratio;
 double max_paid_tax; 
 double min_paid_tax; 
 double Etaxable;
 double receipt;
 double Eadd;
 double E; 
 vector<double> x;
 vector<double> yati;
 vector<double> ymat;
 vector<double> yout;
 double dx_target; 
 double x_target = -0.0001; 
 double x_deviation; 
 double TD,f;
 f = 0.01*inc_percent_;
     for(int i=0; i<percentile_variable_.size(); i++)
     {
     if( percentile_variable_.at(i) < 0.0 ) break;
// -----------------------------------------------------------------------------
     E = scale*rank_ordered_earnings_.at(i);                           // => AGI
     Eadd = scale*transfer_density_.at(i);
     pre_tax_income = E + Eadd;
     min_paid_tax = scale*least_tax_density_.at(i); 
     max_after_tax_income = pre_tax_income - min_paid_tax;    // => key quantity
// -----------------------------------------------------------------------------
//   TD = MIN[ (BD + ID), MAX[BD,MD] , MP*AGI ]                  for victory tax
//   TI = taxable income: Note that TI = GTR + max(0, AGI - TD)  for victory tax
     TD = f*E;                             // BD + ID --> BD_ and MP*AGI --> f*E
       if( TD > Emin_deduction_ )       // Emin_deduction =0 for progressive tax
       {     // ^^^^^^^^^^^^^^^-------------------------> = BD_  for victory tax
       TD = Emin_deduction_;           // => taking no itemized deductions: ID=0
       }
     Etaxable = E - TD;                 
     if( Etaxable < 0.0 ) Etaxable = 0.0;     // => TI_max = maximum possible TI
     max_paid_tax = tax_rate_.at(i)*(ON_*Eadd + Etaxable);  //=> tax_rate*TI_max
     min_after_tax_income = pre_tax_income - max_paid_tax;    // => key quantity
// -----------------------------------------------------------------------------
     receipt = receipt_T_density_.at(i)          // (1 - ON_*r)*transfer_density
             + receipt_D_density_.at(i)          // r*itemized_deduction_density
             + receipt_I_density_.at(i);     // r*capital_loss_deduction_density
     receipt *= scale;                   // an estimate based on deduction model
     subsidie_ratio = receipt/pre_tax_income;        // self-consistent estimate
     x_deviation = x_target - percentile_variable_.at(i); 
       if( x_deviation < 0.0 ) 
       {
       x.push_back( percentile_variable_.at(i) );
       yati.push_back(max_after_tax_income);
       ymat.push_back(min_after_tax_income);
       yout.push_back(subsidie_ratio);
       update(x_target); 
          if( x_target < percentile_variable_.at(i) ) 
          {
          x_target = percentile_variable_.at(i) + 0.00001; 
          }
       } 
     }
 x.push_back(100.000000000);
 yati.push_back(max_after_tax_income);         // independent of deduction model
 ymat.push_back(min_after_tax_income);         // independent of deduction model
 yout.push_back(subsidie_ratio);
// -------------------------------------------------------- write subset of data
    for(int i=0; i<x.size(); i++)
    {
    out_ati << x.at(i) << "  " << yati.at(i) << endl;
    output  << x.at(i) << "  " << yout.at(i) << endl;
    }
 output.close();
 out_ati.close();
 std::string test = "xxxxxxxx";                             // <= does not exist
 if( mn_ati_id_ == test ) return; 
// ---------------------------------------------------- minimum after tax income
 filename = name_ + info_ + mn_ati_id_;
 out_mat.open( filename.c_str() , fstream::out );
    for(int i=0; i<x.size(); i++)
    {
    out_mat << x.at(i) << "  " << ymat.at(i) << endl;
    }
 out_mat.close();
}

void TaxModel::out_g_tr_by_population() 
{
// --------------------------------------------------------------------- purpose
// Calculates the government transfer in real dollars required to eliminate 
// poverty. Note that the minimum (minimum after tax income) should be set at
// the poverty level. However, different tax models can set this assistance line
// without reference to the poverty line. As such, a self-consistent calculation
// can be done to set poverty line to be the same as the minimum of the minimum
// after tax income.  
// Minimal assistance can also be calculated as transfer plus how much a 
// households keeps due to minimum deductions (no itemized deductions and no 
// capital loss deductions). This minimal assistance is therefore independent 
// of the deduction model, and only depends on the tax model. 
// ----------------------------------------------------------------- file set up
 std::string filename;
 fstream output,out_mgva;
 filename = name_ + info_ + kudos_id_;
 output.open( filename.c_str() , fstream::out );
 double scale = 100.0/N_earners_; 
 double receipt_gtr;                       // receiving government transfer only
 double receipt_gtr_ded;       // receiving transfer & tax savings on deductions
 vector<double> x;
 vector<double> ygtr;
 vector<double> ygva;
 double dx_target; 
 double x_target = -0.0001; 
 double x_deviation; 
     for(int i=0; i<percentile_variable_.size(); i++)
     {
     if( percentile_variable_.at(i) < 0.0 ) break;
// --------------------------------------------------------- government transfer
     receipt_gtr = scale*receipt_T_density_.at(i);
// ----------------------------------------- estimated tax savings on deductions
     receipt_gtr_ded = receipt_gtr + scale*receipt_D_density_.at(i); 
     x_deviation = x_target - percentile_variable_.at(i); 
        if( x_deviation < 0.0 ) 
        {
        x.push_back( percentile_variable_.at(i) );
        ygtr.push_back(receipt_gtr);
        ygva.push_back(receipt_gtr_ded);
        update(x_target); 
           if( x_target < percentile_variable_.at(i) ) 
           {
           x_target = percentile_variable_.at(i) + 0.00001; 
           }
        } 
      }
 x.push_back(100.000000000);
 ygtr.push_back(receipt_gtr);
 ygva.push_back(receipt_gtr_ded);
// -------------------------------------------------------- write subset of data
    for(int i=0; i<x.size(); i++)
    {
    output   << x.at(i) << "  " << ygtr.at(i) << endl;
    }
 output.close();
 std::string test = "xxxxxxxx";                             // <= does not exist
 if( mingt_id_ == test ) return; 
 filename = name_ + info_ + mingt_id_;
 out_mgva.open( filename.c_str() , fstream::out );       // just for victory tax
// -------------------------------------------------------- write subset of data
    for(int i=0; i<x.size(); i++)
    {
    out_mgva << x.at(i) << "  " << ygva.at(i) << endl;
    }
 out_mgva.close();
}

void TaxModel::out_gv_tr_lorenz_curve() 
{
// --------------------------------------------------------------------- purpose
// Assuming a representative deduction model, which is just an estimate of a 
// logical scenario, Lorenz curve can be constructed for how total government 
// assistance is spread throughout the population. 
// --------------------------------------------- output transfer in real dollars
 std::string filename;
 fstream output;
 filename = name_ + info_ + gt_lz_id_;
 output.open( filename.c_str() , fstream::out );
 double gini = 0.0;
 double dtemp;
 double sum = 0.0; 
 vector<double> x;
 vector<double> yout;
 double dx_target; 
 double x_target = -0.0001; 
 double receipt_density; 
 double x_deviation; 
     for(int i=0; i<percentile_variable_.size(); i++)
     {
     if( percentile_variable_.at(i) < 0.0 ) break;
     receipt_density = receipt_T_density_.at(i)
                     + receipt_D_density_.at(i)
                     + receipt_I_density_.at(i);  
     sum += receipt_density*d_percentile_variable_.at(i); 
     dtemp = 100.0*sum/net_subsidy_; 
     gini += (percentile_variable_.at(i) - dtemp)*d_percentile_variable_.at(i);
//   output << percentile_variable_.at(i) << "  " << dtemp << endl; 
     x_deviation = x_target - percentile_variable_.at(i); 
         if( x_deviation < 0.0 ) 
         {
         x.push_back( percentile_variable_.at(i) );
         yout.push_back(dtemp);
         update(x_target); 
            if( x_target < percentile_variable_.at(i) ) 
            {
            x_target = percentile_variable_.at(i) + 0.00001; 
            }
         } 
     }
 x.push_back(100.000000000);
 yout.push_back(dtemp);
// -------------------------------------------------------- write subset of data
    for(int i=0; i<x.size(); i++)
    {
    output  << x.at(i) << "  " << yout.at(i) << endl;
    }
 gini /= 5000.0;
 if( (gini < 0.0) && (gini > -0.025) ) gini = 0.0; 
 output << "# Gini coefficient = " << gini << endl;
 output.close();
}

 void TaxModel::out_g_transfer_brckets()     // includes assistance + deductions
{
// --------------------------------------------------------------------- purpose 
// Breakdown the components of government assistance into tax brackets. 
// Assistance components comprise of three catagories: 
// (government transfers)
// (basic + itemized deductions)
// (capital loss deductions) 
// total combines all three catagories. 
// ----------------------------------------------------------- setup file output
 std::string filename;
 fstream output;
 filename = name_ + info_ + Q6_gt_id_;
 output.open( filename.c_str() , fstream::out );
// -------------------------------------------------------------------- brackets
 double receipt_density;
 double sum_bin1 = 0.0;
 double sum_bin2 = 0.0;
 double sum_bin3 = 0.0;
 double sum_bin4 = 0.0;
 double sum_bin5 = 0.0;
 double sum_bin6 = 0.0;
// -------------------------
 double sum_T_bin1 = 0.0;
 double sum_T_bin2 = 0.0;
 double sum_T_bin3 = 0.0;
 double sum_T_bin4 = 0.0;
 double sum_T_bin5 = 0.0;
 double sum_T_bin6 = 0.0;
// -------------------------
 double sum_D_bin1 = 0.0;
 double sum_D_bin2 = 0.0;
 double sum_D_bin3 = 0.0;
 double sum_D_bin4 = 0.0;
 double sum_D_bin5 = 0.0;
 double sum_D_bin6 = 0.0;
// -------------------------
 double sum_I_bin1 = 0.0;
 double sum_I_bin2 = 0.0;
 double sum_I_bin3 = 0.0;
 double sum_I_bin4 = 0.0;
 double sum_I_bin5 = 0.0;
 double sum_I_bin6 = 0.0;
// -------------------------
    for(int i=0; i<percentile_variable_.size(); i++)
    {
    if( percentile_variable_.at(i) < 0.0 ) break;
    receipt_density = receipt_T_density_.at(i)
                    + receipt_D_density_.at(i)
                    + receipt_I_density_.at(i);  
       if( percentile_variable_.at(i) < 20.0 ) 
       {
       sum_T_bin1 += receipt_T_density_.at(i)*d_percentile_variable_.at(i); 
       sum_D_bin1 += receipt_D_density_.at(i)*d_percentile_variable_.at(i); 
       sum_I_bin1 += receipt_I_density_.at(i)*d_percentile_variable_.at(i); 
       sum_bin1   += receipt_density*d_percentile_variable_.at(i); 
       }
       else if( percentile_variable_.at(i) < 40.0 ) 
       {
       sum_T_bin2 += receipt_T_density_.at(i)*d_percentile_variable_.at(i); 
       sum_D_bin2 += receipt_D_density_.at(i)*d_percentile_variable_.at(i); 
       sum_I_bin2 += receipt_I_density_.at(i)*d_percentile_variable_.at(i); 
       sum_bin2   += receipt_density*d_percentile_variable_.at(i); 
       }
       else if( percentile_variable_.at(i) < 60.0 ) 
       {
       sum_T_bin3 += receipt_T_density_.at(i)*d_percentile_variable_.at(i); 
       sum_D_bin3 += receipt_D_density_.at(i)*d_percentile_variable_.at(i); 
       sum_I_bin3 += receipt_I_density_.at(i)*d_percentile_variable_.at(i); 
       sum_bin3   += receipt_density*d_percentile_variable_.at(i); 
       }
       else if( percentile_variable_.at(i) < 80.0 ) 
       {
       sum_T_bin4 += receipt_T_density_.at(i)*d_percentile_variable_.at(i); 
       sum_D_bin4 += receipt_D_density_.at(i)*d_percentile_variable_.at(i); 
       sum_I_bin4 += receipt_I_density_.at(i)*d_percentile_variable_.at(i); 
       sum_bin4   += receipt_density*d_percentile_variable_.at(i); 
       }
       else
       {
       sum_T_bin5 += receipt_T_density_.at(i)*d_percentile_variable_.at(i); 
       sum_D_bin5 += receipt_D_density_.at(i)*d_percentile_variable_.at(i); 
       sum_I_bin5 += receipt_I_density_.at(i)*d_percentile_variable_.at(i); 
       sum_bin5   += receipt_density*d_percentile_variable_.at(i); 
       }
       if( percentile_variable_.at(i) >= 99.0 )
       {
       sum_T_bin6 += receipt_T_density_.at(i)*d_percentile_variable_.at(i); 
       sum_D_bin6 += receipt_D_density_.at(i)*d_percentile_variable_.at(i); 
       sum_I_bin6 += receipt_I_density_.at(i)*d_percentile_variable_.at(i); 
       sum_bin6   += receipt_density*d_percentile_variable_.at(i); 
       }
    }
// ---------------------------------------------------------------------- output
 double sum = 1.0e-5;
 sum += sum_bin1;
 sum += sum_bin2;
 sum += sum_bin3;
 sum += sum_bin4;
 sum += sum_bin5;
// -------------------------------------------------------------- sanity check 2
   if( fabs( sum - net_subsidy_ ) > 1000 )    // typically within 1.0E-8 % error
   {
   cout << "ERROR: sanity check 2 " << endl;
   cout << "       total subsidy sum is not equal to net_subsidy_ " << endl; 
   cout << "  sum of subsidy modes = " << sum << endl;
   cout << "          net_subsidy_ = " << net_subsidy_ << endl;
   exit(-1);
   }
 output << "#                 total government subsidy = " << net_subsidy_ << endl;
 double p1 = 100.0*sum_bin1/sum; 
 double p2 = 100.0*sum_bin2/sum; 
 double p3 = 100.0*sum_bin3/sum; 
 double p4 = 100.0*sum_bin4/sum; 
 double p5 = 100.0*sum_bin5/sum; 
 double p6 = 100.0*sum_bin6/sum; 
 output << "# ---------------------------------------------- brackets " << endl;
 output << "# 10,30,50,70,90 => 20% brackets    110 => top 1% bracket " << endl; 
 output << "# 10=>  0-20%  <gov assistance type>  % of the pie "        << endl; 
 output << "# 30=> 20-40%  <gov assistance type>  % of the pie "        << endl; 
 output << "# etc ... " << endl;
 output << "# ------------------------------------------ bracket data " << endl; 
 double dtemp = 1.0/(0.20*N_earners_);
 output << " 10 " << dtemp*sum_bin1 << "  " << p1 << endl; // between 0% to 20%
 output << " 30 " << dtemp*sum_bin2 << "  " << p2 << endl; //between 20% to 40%
 output << " 50 " << dtemp*sum_bin3 << "  " << p3 << endl; //between 40% to 60%
 output << " 70 " << dtemp*sum_bin4 << "  " << p4 << endl; //between 60% to 80%
 output << " 90 " << dtemp*sum_bin5 << "  " << p5 << endl; //between 80 to 100%
 dtemp = 1.0/(0.01*N_earners_);
 output << "110 " << dtemp*sum_bin6 << "  " << p6 << endl;         // => top 1%
// ----------------------------------------------------------------------------
 double sum_T = 1.0e-5;
 sum_T += sum_T_bin1;
 sum_T += sum_T_bin2;
 sum_T += sum_T_bin3;
 sum_T += sum_T_bin4;
 sum_T += sum_T_bin5;
 output << "# -------------------------------------------------------" << endl;
 output << "#                      government transfer = " << sum_T << endl;
 p1 = 100.0*sum_T_bin1/sum_T; 
 p2 = 100.0*sum_T_bin2/sum_T; 
 p3 = 100.0*sum_T_bin3/sum_T; 
 p4 = 100.0*sum_T_bin4/sum_T; 
 p5 = 100.0*sum_T_bin5/sum_T; 
 p6 = 100.0*sum_T_bin6/sum_T; 
 output << "# ------------------------------------------ bracket data " << endl; 
 dtemp = 1.0/(0.20*N_earners_);
 output << " 10 " << dtemp*sum_T_bin1 << "  " << p1 << endl; // between 0% to 20%
 output << " 30 " << dtemp*sum_T_bin2 << "  " << p2 << endl; //between 20% to 40%
 output << " 50 " << dtemp*sum_T_bin3 << "  " << p3 << endl; //between 40% to 60%
 output << " 70 " << dtemp*sum_T_bin4 << "  " << p4 << endl; //between 60% to 80%
 output << " 90 " << dtemp*sum_T_bin5 << "  " << p5 << endl; //between 80 to 100%
 dtemp = 1.0/(0.01*N_earners_);
 output << "110 " << dtemp*sum_T_bin6 << "  " << p6 << endl;         // => top 1%
// ----------------------------------------------------------------------------
 double sum_D = 1.0e-5;
 sum_D += sum_D_bin1;
 sum_D += sum_D_bin2;
 sum_D += sum_D_bin3;
 sum_D += sum_D_bin4;
 sum_D += sum_D_bin5;
 output << "# -------------------------------------------------------" << endl;
    if( sum_D < 0.01 )
    {
    sum_D = 0.0;
    }
 output << "#    estimated tax savings from deductions = " << sum_D << endl;
 p1 = 100.0*sum_D_bin1/sum_D; 
 p2 = 100.0*sum_D_bin2/sum_D; 
 p3 = 100.0*sum_D_bin3/sum_D; 
 p4 = 100.0*sum_D_bin4/sum_D; 
 p5 = 100.0*sum_D_bin5/sum_D; 
 p6 = 100.0*sum_D_bin6/sum_D; 
 output << "# ------------------------------------------ bracket data " << endl; 
 dtemp = 1.0/(0.20*N_earners_);
 output << " 10 " << dtemp*sum_D_bin1 << "  " << p1 << endl; // between 0% to 20%
 output << " 30 " << dtemp*sum_D_bin2 << "  " << p2 << endl; //between 20% to 40%
 output << " 50 " << dtemp*sum_D_bin3 << "  " << p3 << endl; //between 40% to 60%
 output << " 70 " << dtemp*sum_D_bin4 << "  " << p4 << endl; //between 60% to 80%
 output << " 90 " << dtemp*sum_D_bin5 << "  " << p5 << endl; //between 80 to 100%
 dtemp = 1.0/(0.01*N_earners_);
 output << "110 " << dtemp*sum_D_bin6 << "  " << p6 << endl;         // => top 1%
// ----------------------------------------------------------------------------
 double sum_I = 1.0e-5;
 sum_I += sum_I_bin1;
 sum_I += sum_I_bin2;
 sum_I += sum_I_bin3;
 sum_I += sum_I_bin4;
 sum_I += sum_I_bin5;
 output << "# -------------------------------------------------------" << endl;
 output << "# estimated capital loss deduction savings = " << sum_I << endl;
 p1 = 100.0*sum_I_bin1/sum_I; 
 p2 = 100.0*sum_I_bin2/sum_I; 
 p3 = 100.0*sum_I_bin3/sum_I; 
 p4 = 100.0*sum_I_bin4/sum_I; 
 p5 = 100.0*sum_I_bin5/sum_I; 
 p6 = 100.0*sum_I_bin6/sum_I; 
 output << "# ------------------------------------------ bracket data " << endl; 
 dtemp = 1.0/(0.20*N_earners_);
 output << " 10 " << dtemp*sum_I_bin1 << "  " << p1 << endl; // between 0% to 20%
 output << " 30 " << dtemp*sum_I_bin2 << "  " << p2 << endl; //between 20% to 40%
 output << " 50 " << dtemp*sum_I_bin3 << "  " << p3 << endl; //between 40% to 60%
 output << " 70 " << dtemp*sum_I_bin4 << "  " << p4 << endl; //between 60% to 80%
 output << " 90 " << dtemp*sum_I_bin5 << "  " << p5 << endl; //between 80 to 100%
 dtemp = 1.0/(0.01*N_earners_);
 output << "110 " << dtemp*sum_I_bin6 << "  " << p6 << endl;         // => top 1%
 output.close();
};

void TaxModel::out_effective_tax_rate()
{
// -------------------------------------------------------- output effective tax
 std::string filename;
 fstream output;
 filename = name_ + info_ + ef_tax_id_;
 output.open( filename.c_str() , fstream::out );
 double effective_tax_rate;
 double pretax_income_density; 
 vector<double> x;
 vector<double> yout;
 double dx_target; 
 double x_target = -0.0001; 
 double x_deviation; 
    for(int i=0; i<percentile_variable_.size(); i++)
    {
    if( percentile_variable_.at(i) < 0.0 ) break;
    pretax_income_density = rank_ordered_earnings_.at(i) 
                          + transfer_density_.at(i);
    effective_tax_rate = 100.0*least_tax_density_.at(i)/pretax_income_density;
//  output << percentile_variable_.at(i) << "  " << effective_tax_rate << endl;
    x_deviation = x_target - percentile_variable_.at(i); 
       if( x_deviation < 0.0 ) 
       {
       x.push_back( percentile_variable_.at(i) );
       yout.push_back(effective_tax_rate);
       update(x_target); 
          if( x_target < percentile_variable_.at(i) ) 
          {
          x_target = percentile_variable_.at(i) + 0.00001; 
          }
       } 
    }
 x.push_back(100.000000000);
 yout.push_back(effective_tax_rate);
// -------------------------------------------------------- write subset of data
    for(int i=0; i<x.size(); i++)
    {
    output  << x.at(i) << "  " << yout.at(i) << endl;
    }
 output.close();
}

void TaxModel::out_maximum_tax_rate()
{
// NOTE DEPENDENCIES that is implied for running this subroutine.
// This subroutine should only be run for the victory tax system. 
// Do not run for progressive tax system, because it has no deductibles. 
// ----------------- output maximum tax rate if only standard deduction is taken
 std::string filename;
 fstream output;
 filename = name_ + info_ + "_mtr.tax"; 
 output.open( filename.c_str() , fstream::out );
 double scale = 100.0/N_earners_; 
 double maximum_tax_rate;
 double pretax_income;
 double Eadd; 
 double E,Etaxable; 
 double f,TD;
 double tax_paid; 
 vector<double> x;
 vector<double> yout;
 double dx_target; 
 double x_target = -0.0001; 
 double x_deviation; 
 f = 0.01*inc_percent_;
    for(int i=0; i<percentile_variable_.size(); i++)
    {
    if( percentile_variable_.at(i) < 0.0 ) break;
    E = scale*rank_ordered_earnings_.at(i);
    TD = f*E;                     //  TD = MIN[ (BD + ID), MAX[BD,MD] , MP*AGI ]
       if( TD > BD_ ) 
       {
       TD = BD_;
       }
    Etaxable = E - TD;                   // => with no itemized deductions: ID=0       
    if( Etaxable < 0.0 ) Etaxable = 0.0;
    Eadd = scale*transfer_density_.at(i); 
    pretax_income = E + Eadd; 
    tax_paid = same_tax_rate_*(ON_*Eadd + Etaxable); 
    maximum_tax_rate = 100.0*tax_paid/pretax_income;
//  output << percentile_variable_.at(i) << "  " << maximum_tax_rate << endl;
    x_deviation = x_target - percentile_variable_.at(i); 
       if( x_deviation < 0.0 ) 
       {
       x.push_back( percentile_variable_.at(i) );
       yout.push_back(maximum_tax_rate);
       update(x_target); 
          if( x_target < percentile_variable_.at(i) ) 
          {
          x_target = percentile_variable_.at(i) + 0.00001; 
          }
       } 
    }
 x.push_back(100.000000000);
 yout.push_back(maximum_tax_rate);
// -------------------------------------------------------- write subset of data
    for(int i=0; i<x.size(); i++)
    {
    output  << x.at(i) << "  " << yout.at(i) << endl;
    }
 output.close();
}

void TaxModel::out_paid_by_population()                         // <= taxes paid 
{
// ----------------------------------------------------- output tax Lorenz curve
 std::string filename;
 fstream output;
 filename = name_ + info_ + burden_id_;
 output.open( filename.c_str() , fstream::out );
 output << "# expected tax revenue collected = " << tax_revenue_ << endl;
// ----------------------------------------------------------- output tax values 
 double tax; 
 double sum = 0.0;
 vector<double> x;
 vector<double> yout;
 vector<double> ysum;
 double dx_target; 
 double x_target = -0.0001; 
 double x_deviation; 
    for(int i=0; i<percentile_variable_.size(); i++)
    {
    if( percentile_variable_.at(i) < 0.0 ) break;
    sum += least_tax_density_.at(i)*d_percentile_variable_.at(i); 
    tax = 100.0*least_tax_density_.at(i)/N_earners_;
//  output << percentile_variable_.at(i) << "  " << tax << "  " << sum << endl;
    x_deviation = x_target - percentile_variable_.at(i); 
       if( x_deviation < 0.0 ) 
       {
       x.push_back( percentile_variable_.at(i) );
       yout.push_back(tax);
       ysum.push_back(sum);
       update(x_target); 
          if( x_target < percentile_variable_.at(i) ) 
          {
          x_target = percentile_variable_.at(i) + 0.00001; 
          }
       } 
    }
 x.push_back(100.000000000);
 yout.push_back(tax);
 ysum.push_back(sum);
// -------------------------------------------------------- write subset of data
    for(int i=0; i<x.size(); i++)
    {
    output  << x.at(i) << "  " << yout.at(i) << "  " << ysum.at(i) << endl;
    }
output.close();
}

void TaxModel::out_taxes_lorenz_curve()
{
// ----------------------------------------------------- output tax Lorenz curve
 std::string filename;
 fstream output;
 filename = name_ + info_ + lz_tax_id_;
 output.open( filename.c_str() , fstream::out );
 double norm_tot = tax_revenue_;               // tax collected before transfers
 double norm_net = net_revenue_;     // net revenue = tax revenue less transfers
 double delta = 1.0e40;
 double percent_100;
 double gini = 0.0;
 double sum = 0.0; 
 double z; 
 double dtemp; 
 double test;
 vector<double> x;
 vector<double> yout;
 double dx_target; 
 double x_target = -0.0001; 
 double x_deviation; 
    for(int i=0; i<percentile_variable_.size(); i++)
    {
    if( percentile_variable_.at(i) < 0.0 ) break;
    sum += least_tax_density_.at(i)*d_percentile_variable_.at(i); 
    z = 100.0*sum/norm_tot; 
    if( (z > 99.995) && (z < 100.2) ) z = 99.999; 
//  output <<  percentile_variable_.at(i) << "  " << z << endl;
    x_deviation = x_target - percentile_variable_.at(i); 
       if( x_deviation < 0.0 ) 
       {
       x.push_back( percentile_variable_.at(i) );
       yout.push_back(z);
       update(x_target); 
          if( x_target < percentile_variable_.at(i) ) 
          {
          x_target = percentile_variable_.at(i) + 0.00001; 
          }
       } 
    gini += (percentile_variable_.at(i)  - z)*d_percentile_variable_.at(i);
    dtemp = 100.0*sum/norm_net;                   // defines redistribution mark
    test = fabs(dtemp - 100.0);
       if( test < delta )
       {
       delta = test; 
       percent_100 = percentile_variable_.at(i);  // > rest pays for gov transfr
       }
    }
 x.push_back(100.000000000);
 yout.push_back(z);
// -------------------------------------------------------- write subset of data
    for(int i=0; i<x.size(); i++)
    {
    output  << x.at(i) << "  " << yout.at(i) << endl;
    }
// ------------------------------------------------------- basic characteristics
// Up to the percent_100 shows that all net tax revenues are collected, except 
// for the part of the taxes that are used for government transfer. Taxes 
// collected beyond percent_100 indicate the payment of government transfer
// only, which is a conceptal reflection of how much of the wealthiest segment
// of the population is assisting the entire poorest segment of the populaton. 
// -----------------------------------------------------------------------------
 output << "                           " << endl;
 output <<  percent_100 << "  " << 0.0   << endl;
 output <<  percent_100 << "  " << 99.95 << endl;
 gini /= 5000.0;
 if( (gini < 0.0) && (gini > -0.025) ) gini = 0.0; 
 output << "# Gini coefficient = " << gini << endl;
 output.close();
}

void TaxModel::output()
{
// ------------------------------------ market income: independent of tax models
   market_income_by_earnings();
   market_income_by_population();
   market_income_brackets();
// ---------------------------------------- define model independent information
   BL_ = CL_/(1.0 - CL_);                 // must initialize BL_ = CL_/(1 - CL_) 
   initialize_endaid_line();              // define poverty line or end aid line
   initialize_tax_revenue();              // set tax revenue & gov transfer info

// ======================================================== PROGRESSIVE TAX MODEL
   if( flag_prog_tax_ == 1 )
   {
   calc_prog_tax_revenues();  // => sets output info
// ------------------------------=> now process data
   out_effective_tax_rate();
   out_taxes_lorenz_curve(); 
   out_taxes_paid_brckets();
   out_paid_by_population();  // <= taxes paid 
// -------------------------------------------------------- government tansfers
   out_gov_subsidie_ratio();
   out_g_tr_by_population();         // net transfer = (assistance + deduction) 
   out_gv_tr_lorenz_curve();  
   out_g_transfer_brckets(); 
   }

// ========================================================== VICTORY TAX MODEL
   if( flag_victory_ == 1 )
   {
   calc_victoryT_revenues();  // => sets output info 
// ------------------------------=> now process data
   out_effective_tax_rate();
   out_maximum_tax_rate(); 
   out_taxes_lorenz_curve();  
   out_taxes_paid_brckets();
   out_paid_by_population();  // <= taxes paid 
// -------------------------------------------------------- government tansfers
   out_gov_subsidie_ratio();
   out_g_tr_by_population();         // net transfer = (assistance + deduction) 
   out_gv_tr_lorenz_curve();  
   out_g_transfer_brckets(); 
   }
}

double TaxModel::LSE_Kfunction()
{
// *****************************************************************************
   double sum_bin1;       // calculated average earnings for 1st to 5th quintile
   double sum_bin2;
   double sum_bin3;
   double sum_bin4;
   double sum_bin5;
   double sum_bin6;                    // calculated average earnings for top 1% 
   double d1,d2,d3,d4,d5,d6;                  // percent deviations from targets
   double LSE;                                            // least squares error
// *****************************************************************************
   sum_bin1 = 0.0;
   sum_bin2 = 0.0;
   sum_bin3 = 0.0;
   sum_bin4 = 0.0;
   sum_bin5 = 0.0;
   sum_bin6 = 0.0;
// ----------------------------------------------------- for given alpha, kappa 
    for(int i=0; i<percentile_variable_.size(); i++)
    {
    if( percentile_variable_.at(i) < 0.0 ) break;
       if( percentile_variable_.at(i) < 20.0 ) 
       {
       sum_bin1 += rank_ordered_earnings_.at(i)*d_percentile_variable_.at(i); 
       }
       else if( percentile_variable_.at(i) < 40.0 ) 
       {
       sum_bin2 += rank_ordered_earnings_.at(i)*d_percentile_variable_.at(i); 
       }
       else if( percentile_variable_.at(i) < 60.0 ) 
       {
       sum_bin3 += rank_ordered_earnings_.at(i)*d_percentile_variable_.at(i); 
       }
       else if( percentile_variable_.at(i) < 80.0 ) 
       {
       sum_bin4 += rank_ordered_earnings_.at(i)*d_percentile_variable_.at(i); 
       }
       else
       {
       sum_bin5 += rank_ordered_earnings_.at(i)*d_percentile_variable_.at(i); 
       }
       if( percentile_variable_.at(i) >= 99.0 )
       {
       sum_bin6 += rank_ordered_earnings_.at(i)*d_percentile_variable_.at(i); 
       }
    }
// ----------------------------------- calculate error: measured as % deviations
   double dtemp = 1.0/(0.20*N_earners_);
   sum_bin1 *= dtemp;
   sum_bin2 *= dtemp;
   sum_bin3 *= dtemp;
   sum_bin4 *= dtemp;
   sum_bin5 *= dtemp;
   dtemp = 1.0/(0.01*N_earners_);
   sum_bin6 *= dtemp;
   d1 = (sum_bin1 - target1_)/target1_; 
   d2 = (sum_bin2 - target2_)/target2_; 
   d3 = (sum_bin3 - target3_)/target3_; 
   d4 = (sum_bin4 - target4_)/target4_; 
   d5 = (sum_bin5 - target5_)/target5_; 
   d6 = (sum_bin6 - target6_)/target6_; 
// ------------------------------------------ 5% error defines unity of error.
   d1 *= 20.0;                           
   d2 *= 20.0;                           
   d3 *= 20.0;                           
   d4 *= 20.0;                           
   d5 *= 20.0;                           
   d6 *= 20.0;                           
// ------------------
   double e1 = d1*d1;
   double e2 = d2*d2;
   double e3 = d3*d3;
   double e4 = d4*d4;
   double e5 = d5*d5;
   double e6 = d6*d6;
// ------------------
/*
   d1 = (sum_bin1 - target1_); 
   d2 = (sum_bin2 - target2_); 
   d3 = (sum_bin3 - target3_); 
   d4 = (sum_bin4 - target4_); 
   d5 = (sum_bin5 - target5_); 
   d6 = (sum_bin6 - target6_); 
   cout << "d1 = " << d1 << " sum_bin1 = " << sum_bin1 << " target1 = " << target1_ << endl;
   cout << "d2 = " << d2 << " sum_bin2 = " << sum_bin2 << " target2 = " << target2_ << endl;
   cout << "d3 = " << d3 << " sum_bin3 = " << sum_bin3 << " target3 = " << target3_ << endl;
   cout << "d4 = " << d4 << " sum_bin4 = " << sum_bin4 << " target4 = " << target4_ << endl;
   cout << "d5 = " << d5 << " sum_bin5 = " << sum_bin5 << " target5 = " << target5_ << endl;
   cout << "d6 = " << d6 << " sum_bin6 = " << sum_bin6 << " target6 = " << target6_ << endl;
*/
   LSE = ( e1   +  e2   +  e3   +  e4   +  e5   +  e6  )
       + (e1*e1 + e2*e2 + e3*e3 + e4*e4 + e5*e5 + e6*e6); 
   LSE /= 6.0;
   return LSE;
};

void TaxModel::grid_search(double min_a, double max_a, double min_k, double max_k) 
{
// ----------------------------------------------------------- setup file output
 std::string filename;
 fstream output;
 filename = name_ + info_ + "_LSE.dat";
 output.open( filename.c_str() , fstream::out );
// ------------------------------------------------------ set up a 11 x 11 grid
 double d_a = 0.1*(max_a - min_a);
 double d_k = 0.1*(max_k - min_k);

  for(int i=0; i<11; i++)
  {
  alpha_ = min_a + d_a*i; 
     for(int j=0; j<11; j++)
     {
     kappa_ = min_k + d_k*j; 
     calc_Kfunction(); 
     output << alpha_ << "  " << kappa_ << "  " << LSE_Kfunction() << endl; 
     }
  }
 output.close();
};

// ---------------------------------------------------------------- MAIN program
int main(int argc, char **argv)
{
// ----------------------------------- special variables for automation (add on)
   double PL_target;                                     // target poverty level
   int num_PLon;
// *****************************************************************************
   int count_nE;       // number of earners
   int count_aE;       // average earnings (market income)
   int count_tR;       // tax revenue
   int count_tT;       // tax transfer
   int count_ea;       // end aid line in percentage 
   int num_d_ea;       // end aid line in dollar value
   int count_de;       // deduction: maximum earnings percentile 
   int count_cl;       // deduction: maximum capital loss deductions
   int num_d_de;       // deduction: maximum in dollar value
   int count_dp;       // deduction: maximum percentage of income
   int count_sg;       // specified standard deviation of the distribution 
   int count_al;       // specified alpha for K-function distribution 
   int count_ka;       // specified kappa for K-function distribution 
   int count_mi;       // market income model
   int count_Nm;       // base name for output files
   int count_op;       // specified optimization
   int tag_help;       // specifies when user wants help
   int tag_off;        // specifies when government transfers are not taxed
   int flag_opt;       // (-1,1) => (no file specified, file specified) 
   int count_un;       // counts unknown flags

   TaxModel model;
   double ave_market_income; 
   double min_a,max_a;
   double min_k,max_k;
// *****************************************************************************
   count_nE = 0;
   count_aE = 0;
   count_tR = 0;
   count_tT = 0;
   count_ea = 0;
   num_d_ea = 0;
   count_de = 0;
   count_cl = 0;
   num_d_de = 0;
   count_dp = 0;
   count_sg = 0;
   count_al = 0;
   count_ka = 0;
   count_mi = 0;
   count_Nm = 0;
   count_op = 0;
   num_PLon = 0;
   tag_help = 0;
   tag_off = 0;
   count_un = 0;
   flag_opt = -1;                                   // default no file specified
   min_a = 1.0;
   max_a = 2.5; 
   min_k = 0.45;
   max_k = 0.95; 
// ------------------------------------------- read parameters from command line
      for(int j=1; j<argc; j++)
      {
         if( !strncmp(argv[j],"-PLd=",5) )
         {
         num_PLon++;
         PL_target = atof(argv[j]+5);
         }
         else if( !strncmp(argv[j],"-nE=",4) )
         {
         count_nE++;
         double dtemp = atof(argv[j]+4);
         model.set_nE(dtemp);
         }
         else if( !strncmp(argv[j],"-aE=",4) )
         {
         count_aE++;
         double dtemp = atof(argv[j]+4);
         model.set_aE(dtemp);
         }
         else if( !strncmp(argv[j],"-tR=",4) )
         {
         count_tR++;
         double dtemp = atof(argv[j]+4);
         model.set_tR(dtemp);
         }
         else if( !strncmp(argv[j],"-tT=",4) )
         {
         count_tT++;
         double dtemp = atof(argv[j]+4);
         model.set_tT(dtemp);
         }
         else if( !strncmp(argv[j],"-BD=",4) )
         {
         count_ea++;
         double dtemp = atof(argv[j]+4);
         model.set_BD(dtemp);
         }
         else if( !strncmp(argv[j],"-BDd=",5) )
         {
         num_d_ea++;
         double dtemp = atof(argv[j]+5);
         model.set_BDd(dtemp);  
         }
         else if( !strncmp(argv[j],"-MD=",4) )
         {
         count_de++;
         double dtemp = atof(argv[j]+4);
         model.set_MD(dtemp);
         }
         else if( !strncmp(argv[j],"-MDd=",5) )
         {
         num_d_de++;
         double dtemp = atof(argv[j]+5);
         model.set_MDd(dtemp);  
         }
         else if( !strncmp(argv[j],"-CL=",4) )
         {
         count_cl++;
         double dtemp = atof(argv[j]+4);
         dtemp /= 100.0;       // enter percentage
         model.set_CL(dtemp);
         }
         else if( !strncmp(argv[j],"-MP=",4) )
         {
         count_dp++;
         double dtemp = atof(argv[j]+4);
         model.set_MP(dtemp);
         }
         else if( !strncmp(argv[j],"-sig=",4) )
         {
         count_sg++;
         double dtemp = atof(argv[j]+5);
         model.set_sig(dtemp);
         }
         else if( !strncmp(argv[j],"-alp=",4) )
         {
         count_al++;
         double dtemp = atof(argv[j]+5);
         model.set_alp(dtemp);
         }
         else if( !strncmp(argv[j],"-kap=",4) )
         {
         count_ka++;
         double dtemp = atof(argv[j]+5);
//       kappa must be in range: [0,1) but numerically use [0.001,0.999]
         if( dtemp < 0.001 ) dtemp = 0.001; 
         if( dtemp > 0.999 ) dtemp = 0.999; 
         model.set_kap(dtemp);
         }
         else if( !strncmp(argv[j],"-mi=",4) )
         {
            if( !strncmp(argv[j],"-mi=all",7) )
            {
            cout << endl; 
            cout << endl; 
            cout << "This feature not implemented yet " << endl;
            cout << endl; 
            exit(-1); 
            }
         count_mi++;
         std::string model_type(argv[j]+4);  
         model.set_mi(model_type);
         }
         else if( !strncmp(argv[j],"-name=",6) )
         {
         count_Nm++;
         model.set_Nm(argv[j]+6);
         }
         else if( !strncmp(argv[j],"-op",3) )
         {
         std::string mtype = "Kfunction"; 
         model.set_mi(mtype);
         double dtemp = 1.8;
         model.set_alp(dtemp);
         dtemp = 0.8;
         model.set_kap(dtemp);
         count_op++; 
            if( !strncmp(argv[j],"-op=",4) )
            {
               if( strlen(argv[j]) > 4 )
               {
               flag_opt = 1; 
               model.set_fop(argv[j]+4);
               }
            }
         }
         else if( !strncmp(argv[j],"-min_a=",7) )
         {
         min_a = atof(argv[j]+7);
         }
         else if( !strncmp(argv[j],"-max_a=",7) )
         {
         max_a = atof(argv[j]+7);
         }
         else if( !strncmp(argv[j],"-min_k=",7) )
         {
         min_k = atof(argv[j]+7);
         }
         else if( !strncmp(argv[j],"-max_k=",7) )
         {
         max_k = atof(argv[j]+7);
         }
         else if( !strncmp(argv[j],"-off",4) )
         {
         tag_off++; 
         }
         else if( !strncmp(argv[j],"-help",5) )
         {
         tag_help++; 
         }
         else
         {
         count_un++;                                                  // unknown
         }
      }
// ------------------------------------------- write to screen basic information
   if( argc < 2 ) tag_help = 1;                            // => user needs help
   int flag_error = -1;                                           // => no error
   if( tag_help > 0 ) goto write_usage;
// ---------------------------------------------------------- basic error checks
      if( count_nE > 1 ) 
      {
      cout << "ERROR: multiple specifications for # of earners " << endl;
      flag_error = 1;
      }
      if( count_aE > 1 ) 
      {
      cout << "ERROR: multiple specifications for average Earnings " << endl;
      flag_error = 1;
      }
      if( count_tR > 1 ) 
      {
      cout << "ERROR: multiple specifications for tax revenue " << endl;
      flag_error = 1;
      }
      if( count_tT > 1 ) 
      {
      cout << "ERROR: multiple specifications for tax transfer " << endl;
      flag_error = 1;
      }
      if( (num_d_ea + count_ea) > 1 ) 
      {
      cout << "ERROR: multiple specifications for provert line " << endl;
      flag_error = 1;
      }
      if( (num_d_de + count_de) > 1 ) 
      {
      cout << "ERROR: multiple specifications for maximum deduction " << endl;
      flag_error = 1;
      }
      if( count_cl > 1 ) 
      {
      cout << "ERROR: multiple specifications for capital loss rate" << endl;
      flag_error = 1;
      }
      if( count_dp > 1 ) 
      {
      cout << "ERROR: multiple specifications for percentage of income " << endl;
      flag_error = 1;
      }
      if( count_sg > 1 ) 
      {
      cout << "ERROR: multiple specifications for standard deviation " << endl;
      flag_error = 1;
      }
      if( count_al > 1 ) 
      {
      cout << "ERROR: multiple specifications for alpha" << endl;
      flag_error = 1;
      }
      if( count_ka > 1 ) 
      {
      cout << "ERROR: multiple specifications for kappa" << endl;
      flag_error = 1;
      }
      if( count_Nm > 1 ) 
      {
      cout << "ERROR: multiple specifications for name " << endl;
      flag_error = 1;
      }
      if( count_op > 1 ) 
      {
      cout << "ERROR: multiple specifications for optimization " << endl;
      flag_error = 1;
      }
      if( (count_op > 0) && (count_mi > 0) )
      {
      cout << "ERROR: using the -op flag implies Kfunction " << endl;
      cout << "       cannot use -mi option with the -op option " << endl;
      cout << "       cannot use auxilary model parameter options " << endl; 
      flag_error = 1;
      }
      if( count_un > 0 ) 
      {
      cout << "ERROR: unknown command line arguments found" << endl;
      tag_help = 1;
      flag_error = 1;
      }
   write_usage:
      if( tag_help > 0 ) 
      {
      cout << endl;
      cout << "USAGE: " << endl; 
      cout << "--------------------------------------------------------- modes: 0, 1, 2 " << endl; 
      cout << "MODE 0 examples: "                                                         << endl;
      cout << argv[0] << "  -op "                                                         << endl;
      cout << argv[0] << "  -op=data2003.tax  -name=sweep1  "                             << endl;
      cout << argv[0] << "  -op=data2003.tax  -name=sweep2  " 
           << "-min_a=1.1  -max_a=1.6  -min_k=0.5  -max_k=0.8 "                           << endl;
      cout << endl;
      cout << "MODE 1 examples: "                                                         << endl;
      cout << argv[0] << " -name=Kfnc1  " 
           << "-nE=112117000  -aE=75300  -tR=1952929045000  -tT=1098746600000  " 
           << "-mi=Kfunction  -alp=1.15  -kap=0.56  " 
           << "-BD=40  -MD=50  -MP=60  -CL=40 "                                           << endl; 
      cout << argv[0] << " -name=Kfnc1  " 
           << "-nE=112117000  -aE=75300  -tR=1952929045000  -tT=1098746600000  " 
           << "-mi=Kfunction  -alp=1.15  -kap=0.56  " 
           << "-BD=40  -MD=50  -MP=60  -CL=40  -off "                                     << endl; 
      cout << argv[0] << " -name=quad1  " 
           << "-nE=112117000  -aE=75300  -tR=1952929045000  -tT=1098746600000  " 
           << "-mi=quadratic  " 
           << "-BD=40  -MD=50  -MP=60  -CL=20 "                                           << endl; 
      cout << argv[0] << " -name=lgnm1  " 
           << "-nE=112117000  -aE=75300  -tR=1952929045000  -tT=1098746600000  " 
           << "-mi=lognormal  -sig=112700  " 
           << "-BD=40  -MD=50  -MP=60  -CL=0 "                                            << endl; 
      cout << endl;
      cout << "MODE 2 examples: "                                                         << endl;
      cout << argv[0] << " -name=comp1  " 
           << "-nE=112117000  -aE=75300  -tR=1952929045000  -tT=1098746600000  " 
           << "-mi=all  -alp=1.15  -kap=0.56  "                                           << endl;
      cout << endl;
      cout << "---------------------------------------------------- command description " << endl;
      cout << " -name=out   => base name for all files generated "                        << endl; 
      cout << "                                                                IRS data " << endl;
      cout << " -nE=______  => number of earners (normalized households) "                << endl; 
      cout << " -aE=______  => average earnings (market income) from entire population "  << endl; 
      cout << " -tR=______  => total household tax revenue collected from the population" << endl; 
      cout << " -tT=______  => total household tax tranfer back to the population "       << endl; 
      cout << endl;
      cout << "                                             market income distributions " << endl;
      cout << " -mi=______  => density model  and associated auxilary parameter options " << endl; 
      cout << "                uniform "                                                  << endl;
      cout << "                linear "                                                   << endl;
      cout << "                quadratic "                                                << endl;
      cout << "                lognormal   -sig=10000 "                                   << endl;
      cout << "                Kfunction   -alp=2.0   -kap=0.75 "                         << endl;
      cout << " -op=<file>  => optimize Kfunction parameters to fit actual data"          << endl; 
      cout << "                example output file is generated without <file> specified" << endl;
      cout << "                -min_a=1.00  -max_a=2.50 "                                 << endl;
      cout << "                -min_k=0.45  -max_k=0.95 "                                 << endl;
      cout << endl;
      cout << "                                                    tax model properties " << endl;
      cout << " -off        => do not tax government transfers "                          << endl;
      cout << " -MP=______  => maximum deduction by percentage of income "                << endl; 
      cout << " -CL=______  => maximum percent of income claimed as capital loss "        << endl; 
      cout << " -MD=______  => maximum deduction by earnings percentile"                  << endl; 
      cout << " -BD=______  => end aid line by earnings percentile "                      << endl; 
      cout << "--------------------------------------------------------- alternate inputs"<< endl;
      cout << " -MDd=_____  => maximum deduction by dollar value "                        << endl; 
      cout << " -BDd=_____  => end aid line by dollar value"                              << endl; 
      cout << "---------------------------------------------------------- automated input"<< endl;
      cout << " -PLd=_____  => poverty line by dollar value REPLACES -BDd= and -BD="      << endl; 
      cout << "      ^-----------------> no gap "                                         << endl; 
      cout << " Description of automation "                                               << endl;
      cout << " When -PLd is specified, three cases are automatically generated "         << endl;
      cout << "     victory tax with BDd determined internally for specified MDd and MP"  << endl;
      cout << "        flat tax with BDd determined internally for specified MDd and MP=0"<< endl;
      cout << " progressive tax with BDd determined internally for same poverty level"    << endl;
      cout << endl;
      cout << "================================================================== RESULTS"<< endl;
      cout << endl;
cout << "  MODE 0: -op=user_specified_file_name_containing_quintile_data "                << endl;
cout << "          -few_options_to_define_search_grid_range ... "                         << endl;
cout << "          -name=OUT "                                                            << endl;
cout << endl;
cout << "  OUTPUT: default OUT = out "                                                    << endl;
cout << "  grid optimization "                                                            << endl;
cout << "  YIELDS: out_LSE.dat  => 3 columns: alpha  kappa  fitting_error "               << endl;
cout << endl;
cout << "  MODE 1: -mi=user_specified_market_income_distribution "                        << endl;
cout << "          -di=user_specified_deduction_income_percentage "                       << endl;
cout << "          -dE=user_specified_deduction_earnings_percentile "                     << endl;
cout << "          -many_other_options_possible ... "                                     << endl;
cout << "          -name=OUT "                                                            << endl;
cout << endl;
cout << "  OUTPUT: default OUT = out "                                                    << endl;
cout << "  Income properties "                                                            << endl;
cout << "  YIELDS: out_xyz.inc  <= xyz specifies type of data in file "                   << endl;
cout << "          -----------     xyz = Pmi, Pmav, Pmxx, rmi, mLz, miQ "                 << endl;
cout << "          out_Pmi.inc  => Pmi(E) = population density by market income "         << endl;
cout << "          out_Pmav.inc => Pmi(E) marked at average-earnings "                    << endl;
cout << "          out_Pmxx.inc => Pmi(E) marked at earning percentiles 20,40,50,60,80,99"<< endl;
cout << "          out_rmi.inc  => rank-ordered market income earnings "                  << endl;
cout << "          out_mLz.inc  => market income Lorenz curve "                           << endl;
cout << "          out_miQ.inc  => market income for 5-quintiles and top 1% quantile "    << endl;
cout << endl;
cout << "  Deduction properties "                                                         << endl;
cout << "  YIELDS: out_xyz.ded  <= xyz specifies type of data in file "                   << endl;
cout << "          -----------     xyz = cld, edv, clr, dir "                             << endl;
cout << "          out_cld.ded  => capital loss deduction estimate (model independent)"   << endl;
cout << "          out_edv.ded  => estimated deduction within victory tax model "         << endl;
cout << "          out_tvt.ded  => estimated total deduction within victory tax model "   << endl;
cout << "          out_clr.ded  => capital loss deduction to income ratio "               << endl;
cout << "          out_dir.ded  => (BD + ID)-est to income ratio in victory tax model"    << endl;
cout << "          out_tvr.ded  => total deduction to income ratio in victory tax model"  << endl;

cout << endl;
cout << "  Tax properties "                                                               << endl;
cout << "  YIELDS: out_xyz.tax  <= xyz specifies type of data in file "                   << endl;
cout << "          -----------     xyz = pti, let,      ptp, pLz, ptQ, Ppt, Cpt "         << endl;
cout << "          out_pti.tax  => progressive tax information "                          << endl;
cout << "          out_let.tax  => linear effective tax rate for population percentile "  << endl;
cout << "          out_ptp.tax  => progressive tax paid by population percentile "        << endl;
cout << "          out_pLz.tax  => progressive tax Lorenz curve  "                        << endl;
cout << "          out_ptQ.tax  => progressive tax for 5-quintiles and top 1% quantile "  << endl;
cout << "          out_Ppt.tax  => pdf of population paying progressive tax rate "        << endl;
cout << "          out_Cpt.tax  => pdf of population paying progressive tax rate "        << endl;
cout << "          -----------     xyz = vti, etr, mtr, vtp, vLz, vtQ, Pvt, Cvt "<< endl;
cout << "          out_vti.tax  => victory tax information "                              << endl;
cout << "          out_etr.tax  => effective victory tax rate for population percentile " << endl;
cout << "          out_mtr.tax  => maximum tax rate using S.D. for population percentile" << endl;
cout << "          out_vtp.tax  => victory tax paid by population percentile "            << endl;
cout << "          out_vLz.tax  => victory tax Lorenz curve  "                            << endl;
cout << "          out_vtQ.tax  => victory tax for 5-quintiles and top 1% quantile "      << endl;
cout << "          out_Pvt.tax  => pdf of population paying etr for victory tax "         << endl;
cout << "          out_Cvt.tax  => cdf of population paying etr for victory tax "         << endl;
/* %%%%%%%%%%%%%   not implemented due to it being complicated to resort data, but it is possible!
cout << "          out_Ppav.tax => Ppt(E) marked at average progressive tax "             << endl;
cout << "          out_Ppxx.tax => Ppt(E) marked at earning percentiles 20,40,50,60,80,99"<< endl;
cout << "          out_rpt.inc  => rank-ordered progressive taxes "                       << endl;
cout << "          out_Pfav.tax => Ppt(E) marked at average victory tax "                 << endl;
cout << "          out_Pfxx.tax => Ppt(E) marked at earning percentiles 20,40,50,60,80,99"<< endl;
%%%%%%%%%%%%%%%% */
cout << endl;
cout << "  Ater tax income properties "                                                   << endl;
cout << "  YIELDS: out_xyz.ati  <= xyz specifies different types of after tax income "    << endl;
cout << "          -----------     xyz = lpt, vtf, vts "                                  << endl;
cout << "          out_lpt.ati  => linear progressive tax using rank-ordered percentiles" << endl;
cout << "          out_vtf.ati  => victory tax with full-deduction     => maximum income" << endl;
cout << "          out_vts.ati  => victory tax with standard-deduction => minimum income" << endl;
/* %%%%%%%%%%%%%   not implemented due to it being complicated to resort data, but it is possible!
cout << "          -----------     xyz = Pat, Paav, Paxx, rai, aLz, aiQ "                 << endl;
cout << "          out_Pat.ati  => Pat(E) = population density by after-tax income "      << endl;
cout << "          out_Paav.ati => Pat(E) marked at average-earnings "                    << endl;
cout << "          out_Paxx.ati => Pat(E) marked at earning percentiles 20,40,50,60,80,99"<< endl;
cout << "          out_rai.ati  => rank-ordered after-tax income earnings "               << endl;
cout << "          out_aLz.ati  => after-tax income Lorenz curve "                        << endl;
cout << "          out_aiQ.ati  => after-tax income for 5-quintiles and top 1% quantile " << endl;
cout << "          out_rft.inc  => rank-ordered victory taxes "                           << endl;
%%%%%%%%%%%%%%%% */

cout << endl;
cout << "  Government assistance properties "                                             << endl;
cout << "                          vgt = transfer only "                                  << endl;
cout << "                          vga = transfer + deductions only "                     << endl;
cout << "                          vsr = subsidy = transfer + deductions + capital losses"<< endl;
cout << "  YIELDS: out_xyz.gva  <= xyz specifies type of government assistance "          << endl;
cout << "          -----------     xyz = vgt, vsr, vLz, vtQ, vga   for victory tax "      << endl;
cout << "          out_vgt.gva  => government transfer by population percentile "         << endl;
cout << "          out_vsr.gva  => subsidy ratio = (subsidy)/(pre_tax_income) "           << endl;
cout << "          out_vLz.gva  => Lorenz curve for government subsidy "                  << endl;
cout << "          out_vtQ.gva  => % tax collected from 5-quintiles & top 1% quantile "   << endl;
cout << "          out_vga.gva  => government assistance by population percentile "       << endl; 
cout << "          -----------     xyz = pgt, psr, pLz, ptQ    for progressive tax "      << endl;
cout << "          out_pgt.gva  => government transfer by population percentile "         << endl;
cout << "          out_psr.gva  => subsidy ratio = (subsidy)/(pre_tax_income) "           << endl;
cout << "          out_pLz.gva  => Lorenz curve for government subsidy "                  << endl;
cout << "          out_ptQ.gva  => % tax collected from 5-quintiles & top 1% quantile "   << endl;
cout << "          out_pgt.gva  =  out_vga.gva since progressive tax has no deductibles " << endl;

cout << endl;
cout << "  MODE 2: -mi=all "                                                              << endl;
cout << "          -few_required_options_for_k_function_model ... "                       << endl;
cout << "          -name=OUT "                                                            << endl;
cout << endl;
cout << "  OUTPUT: default OUT = out "                                                    << endl;
cout << endl;
cout << " PROCESS: starting with a k_function model, generates all comparisons "          << endl;
cout << "          (sig for log normal distr) = 2/3 x (sig for k-function distr) "        << endl;
cout << endl;
cout << "  YIELDS: Same results as mode 1, but OUT changes to the following form: "       << endl;
cout << "          out --> out_INCM_MP_MD_xyz.inc   and   out_IM_MP_MD_xyz.tax "          << endl;
cout << "                  INCM = income model: {unif, linr, quad, logn, kfnc} "          << endl;
cout << "                  MP   = integer percentage-of-income ranging from 00 to 99 "    << endl;
cout << "                  MD   = integer earnings  percentile ranging from 00 to 99 "    << endl;
cout << "          ** all income models and many combinations of MP, MD are explored "    << endl;
cout << endl;
cout << "          out.cmd  => records command line that generated the results "          << endl;
      cout<< "==========================================================================="<< endl;
         if( flag_error > 0 ) 
         {
         cout << "ERRORS: Command line arguments have some type of error: " << endl;
         cout << "        Scroll up to begining of USAGE to read error messages. " << endl;
         cout << "        Program terminated. " << endl;
         }
      cout << endl;
      exit(-1); 
      }
      if( flag_error > 0 ) 
      {
      cout << "---------------------> At least one error was detected " << endl;
      exit(-1); 
      }
   if( tag_off > 0 ) model.turnoff_taxing_transfers();
   model.specialize_name(); 
// ------------------------------------------- write out summary of calculations
   {
   std::string filename;
   fstream summary;
   filename = model.get_Nm() + model.get_if() + ".cmd";
   summary.open( filename.c_str() , fstream::out );
      for(int j=0; j<argc; j++)
      {
      summary << argv[j] << " ";
      }
   summary << endl; 
   summary.close();
   }
// ------------------------------------------ optimation template file generated
      if( (count_op > 0) && (flag_opt < 0) ) 
      {
      cout << endl;
      std::string filename;
      fstream std_format;
      filename = model.get_fo();
      cout << "Sample input file (2003 IRS data on US taxes) is created " << endl;
      cout << "file name = " << filename << endl;
      cout << "Rename file to something that makes sense. Edit as necessary." << endl;
      cout << "This file can be processed with successive grid searches " << endl;
      cout << "Not sure?  type: " << argv[0] << "  -help " << endl;
      cout << endl;
      std_format.open( filename.c_str() , fstream::out );
      std_format << "#   112100000   number of earners " << endl;
      std_format << "#   75300       average market income for all earners " << endl;
      std_format << "#   10,30,50,70,90,110 => 1st,2nd,3rd,4th,5th quintiles, top 1%" << endl;
      std_format << "10  7400        " << endl;
      std_format << "30  31000       " << endl;
      std_format << "50  54800       " << endl;
      std_format << "70  85100       " << endl;
      std_format << "90  211000      " << endl;
      std_format << "110 1170100     " << endl;
      std_format.close();
      exit(-1);                                                          // stop
      }
// ---------------------------------------------- read target data for Kfunction 
      if( (count_op > 0) && (flag_opt > 0) ) 
      {
      model.read_tax_data();
      if( model.set_market_income(argc,argv) == false ) exit(-1);
      model.grid_search(min_a,max_a,min_k,max_k); 
      exit(-1);
      }
   if( model.set_market_income(argc,argv) == false ) exit(-1);
   model.Exx_earnings();
   model.std_earnings();
// ----------------------------- determine if one-shot calculation or automation
   int flag_automate = -1;                               // assume no automation
// count_ea = end aid line in percentage 
// num_d_ea = end aid line in dollar value
      if( num_PLon == 1 ) 
      {
         if( (count_ea + num_d_ea) == 0 )
         {
         flag_automate = 1; 
            if( PL_target < 0.0 )
            {
            cout << endl;
            cout << "ERROR: poverty level must be at least 0 " << endl;
            cout << endl;
            exit(-1);
            }
         }
         else
         {
         cout << endl;
         cout << "ERROR: when PLd= is specified BDd= and BD= cannot be."<< endl;
         cout << "       Calculation is aborted."<< endl;
         cout << endl;
         exit(-1);
         }
      }
      else if( num_PLon > 1 ) 
      {
      cout << endl;
      cout << "ERROR: can only specify PLd= once."<< endl;
      cout << "       Calculation is aborted."<< endl;
      cout << endl;
      exit(-1);
      }
// -------------------------------------------------------- perform calculations
      if( flag_automate < 0 )                                     // => one shot
      {
      model.do_victoryT_only();      // the focus of this program is Victory tax
      model.output();
      exit(-1); 
      }
// --------- compare different models only when they have the same poverty level
// ---------------------------------------------------------- perform automation
   cout << "automation in progress: target poverty level = " 
        << PL_target << endl;
   cout << "VICTORY TAX MODEL" << endl;
   cout << endl;
// ----------------------------------------------- define basic variables needed
   double minBDd;                                         // lower limit bracket 
   double tryBDd;                     // current trial value for basid deduction
   double maxBDd;                                         // upper limit bracket 
   double gotPL;                           // current value of the poverty level
   int flag_success;                          // = (-1,1) for (failure, success)
// -------------------------------------------------------- consider victory tax
   model.do_victoryT_only();
   minBDd=PL_target;                             // <= 0% tax rate makes BD = PL 
   maxBDd=3*PL_target;                // <= 2/3 = 66.6% tax rate makes BD = 3*PL 
// ------------------------- searching for a victory tax rate between 0 to 66.6%
   flag_success = -1;                         // assumes no convergence: failure
      for(int j=0; j<25; j++)          // provides more than 0.1 dollar accuracy
      {
      tryBDd = (minBDd + maxBDd)/2;
      model.set_BDd(tryBDd);  
      model.specialize_name(); 
      model.output();
      gotPL = model.get_poverty_level(); 
      cout << "BDd= " << tryBDd << "  PLd= " << gotPL << endl;
         if( fabs(gotPL - PL_target) < 0.1 )                          // success
         {
         flag_success = 1;
         break; 
         }
         else if( gotPL > PL_target )          // => basic deduction is too high
         {
         maxBDd = tryBDd;             // current guess for BD is now the maximum
         }
         else                                   // => basic deduction is too low
         {
         minBDd = tryBDd;             // current guess for BD is now the minimum
         }
// ----------------------------------------------------------------- remove case
         {
         std::stringstream ss;
         ss << "rm " << model.get_Nm() << model.get_if() << "*";
         std::string cmd0 = ss.str();
         system( cmd0.c_str() ); 
         }
      }
// ------------------------------------------------ finished with the iterations
      if( flag_success > 0 )
      {
         {
         std::stringstream ss;
         ss << "mv " << model.get_Nm() << "_BD40*.cmd  " << model.get_Nm() 
            << "_PLd" << PL_target << "_MDd" << model.get_MD() << "_MP" 
            << model.get_MP() << "_CL" << (model.get_CL()*100);
            if( model.get_ON() > 0.5 )
            {
            ss << "_ON.cmd";
            }
            else
            {
            ss << "_OFF.cmd";
            }  
         std::string cmd1 = ss.str();
         system( cmd1.c_str() ); 
         }
      cout << "SUCCESS: Actual poverty level set at " << gotPL << endl;
      cout << endl;
      }
      else
      {
      cout << "FAILED: Requested poverty level could not be achieved" << endl;
      cout << "        Check maximum deduction and percent deduction" << endl;
      cout << endl;
      exit(-1);
      }
   cout << endl;
   cout << "FLAT TAX MODEL" << endl;
   cout << endl;
// ----------------------------------------------------------- consider flat tax
   model.set_MP(0);  
   minBDd=PL_target;                             // <= 0% tax rate makes BD = PL 
   maxBDd=3*PL_target;                // <= 2/3 = 66.6% tax rate makes BD = 3*PL 
// ---------------------------- searching for a flat tax rate between 0 to 66.6%
   flag_success = -1;                         // assumes no convergence: failure
      for(int j=0; j<25; j++)          // provides more than 0.1 dollar accuracy
      {
      tryBDd = (minBDd + maxBDd)/2;
      model.set_BDd(tryBDd);  
      model.specialize_name(); 
      model.output();
      gotPL = model.get_poverty_level(); 
      cout << "BDd= " << tryBDd << "  PLd= " << gotPL << endl;
         if( fabs(gotPL - PL_target) < 0.1 )                          // success
         {
         flag_success = 1;
         break; 
         }
         else if( gotPL > PL_target )          // => basic deduction is too high
         {
         maxBDd = tryBDd;             // current guess for BD is now the maximum
         }
         else                                   // => basic deduction is too low
         {
         minBDd = tryBDd;             // current guess for BD is now the minimum
         }
// ----------------------------------------------------------------- remove case
         {
         std::stringstream ss;
         ss << "rm " << model.get_Nm() << model.get_if() << "*";
         std::string cmd0 = ss.str();
         system( cmd0.c_str() ); 
         }
      }
// ------------------------------------------------ finished with the iterations
      if( flag_success > 0 )
      {
      cout << "SUCCESS: Actual poverty level set at " << gotPL << endl;
      cout << endl;
      }
      else
      {
      cout << "FAILED: Requested poverty level could not be achieved" << endl;
      cout << "        Check maximum deduction and percent deduction" << endl;
      cout << endl;
      exit(-1);
      }
   cout << endl;
   cout << "PROGRESSIVE TAX MODEL" << endl;
   cout << endl;
// ---------------------------------------------------- consider progressive tax
   model.do_prog_tax_only();
   model.set_MP(0);          // like flat tax, progressive tax has no deductions
   model.set_MDd(-1.0);            // reset MDd input parameter to default value
   model.set_MD(0);          // like flat tax, progressive tax has no deductions
   minBDd=PL_target;                             // <= 0% tax rate makes BD = PL 
   maxBDd=3*PL_target;                // <= 2/3 = 66.6% tax rate makes BD = 3*PL 
// --------------------- searching for a progressive tax rate between 0 to 66.6%
   flag_success = -1;                         // assumes no convergence: failure
      for(int j=0; j<25; j++)          // provides more than 0.1 dollar accuracy
      {
      tryBDd = (minBDd + maxBDd)/2;
      model.set_BDd(tryBDd);  
      model.specialize_name(); 
      model.output();
      gotPL = model.get_poverty_level(); 
      cout << "BDd= " << tryBDd << "  PLd= " << gotPL << endl;
         if( fabs(gotPL - PL_target) < 0.1 )                          // success
         {
         flag_success = 1;
         break; 
         }
         else if( gotPL > PL_target )          // => basic deduction is too high
         {
         maxBDd = tryBDd;             // current guess for BD is now the maximum
         }
         else                                   // => basic deduction is too low
         {
         minBDd = tryBDd;             // current guess for BD is now the minimum
         }
// ----------------------------------------------------------------- remove case
         {
         std::stringstream ss;
         ss << "rm " << model.get_Nm() << model.get_if() << "*";
         std::string cmd0 = ss.str();
         system( cmd0.c_str() ); 
         }
      }
// ------------------------------------------------ finished with the iterations
      if( flag_success > 0 )
      {
      cout << "SUCCESS: Actual poverty level set at " << gotPL << endl;
      cout << endl;
      }
      else
      {
      cout << "FAILED: Requested poverty level could not be achieved" << endl;
      cout << "        Check maximum deduction and percent deduction" << endl;
      cout << endl;
      exit(-1);
      }

// ------------------------------------------------------ miscellaneous leftover
/*
      for( double x=0.01; x<2.000000001; x+=0.001)
      {
      cout << x << "  " << gamma_function(x) << endl;
      }
// -------------------------
   double kappa = 1.05; 
      for( double x=-100; x<100.0001; x+=0.1)
      {
      cout << x << "  " << expK(x,kappa) << endl;
      }
*/
}

